package com.blockchain.common.base.util;

import com.aliyuncs.utils.StringUtils;
import com.blockchain.common.base.constant.BaseConstant;
import com.blockchain.common.base.dto.YyyfUserDto;
import org.springframework.data.redis.core.RedisTemplate;

import javax.servlet.http.HttpServletRequest;

/**
 * @author huan.xu
 * @version 1.0
 * @className YyyfUserDto
 * @description
 * @date 2020-05-14 14:09
 */
public class YyyfUserDtoUtils {

    /**
     * @description 从cookies 获取以渔有方用户信息
     * @param [request, redisTemplate]
     * @return com.blockchain.common.base.dto.YyyfUserDto
     **/
    public static YyyfUserDto getYyyfUserDto(HttpServletRequest request, RedisTemplate redisTemplate) {
        YyyfUserDto yyyfUserDto = null;
        String yyyfTtoken = request.getHeader(BaseConstant.YYYF_TOKEN_HEADER);
        if(yyyfTtoken!=null) {
            //如果有token，拿到用户信息,与以渔有方进行绑定
            if (redisTemplate.hasKey(yyyfTtoken)) {
                yyyfUserDto = (YyyfUserDto) redisTemplate.opsForValue().get(yyyfTtoken);
            }
        }
        return yyyfUserDto;
    }


    /**
     * @description 通过userId获取以渔有方用户信息
     * @author huan.xu
     * @date  2020-05-20 15:42:47
     * @param [userId, redisTemplate]
     * @return com.blockchain.common.base.dto.YyyfUserDto
     **/
    public static YyyfUserDto getYyyfUserDtoByUserId(String userId, RedisTemplate redisTemplate) {
        YyyfUserDto yyyfUserDto = null;
        if(StringUtils.isNotEmpty(userId)) {
            //如果有token，拿到用户信息,与以渔有方进行绑定
            String key=BaseConstant.USER_ID_YYYF_USER_KEY.concat(userId);
            if (redisTemplate.hasKey(key)) {
                yyyfUserDto = (YyyfUserDto) redisTemplate.opsForValue().get(key);
            }
        }
        return yyyfUserDto;
    }
}