package com.blockchain.server.btc.controller;

import com.blockchain.common.base.constant.BaseConstant;
import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.common.base.util.SSOHelper;
import com.blockchain.server.base.controller.BaseController;
import com.blockchain.server.btc.common.constants.BtcApplicationConstans;
import com.blockchain.server.btc.controller.api.BtcWalletTransferApi;
import com.blockchain.server.btc.dto.BtcWalletTransferDTO;
import com.blockchain.server.btc.service.BtcWalletTransferService;
import com.github.pagehelper.PageHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Api(BtcWalletTransferApi.MARKET_CONTROLLER_API)
@RestController
@RequestMapping("/walletTransfer")
public class BtcWalletTransferController extends BaseController {
    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private BtcWalletTransferService btcWalletTransferService;


    @ApiOperation(value = BtcWalletTransferApi.GetTransfer.METHOD_API_NAME, notes = BtcWalletTransferApi.GetTransfer.METHOD_API_NOTE)
    @GetMapping("/getTransfer")
    public ResultDTO getTransfer(HttpServletRequest request,
                                 @ApiParam(BtcWalletTransferApi.PAGENUM) @RequestParam(name = "pageNum", defaultValue = BaseConstant.PAGE_DEFAULT_INDEX) Integer pageNum,
                                 @ApiParam(BtcWalletTransferApi.PAGESIZE) @RequestParam(name = "pageSize", defaultValue = BaseConstant.PAGE_DEFAULT_SIZE) Integer pageSize,
                                 @ApiParam(BtcWalletTransferApi.GetTransfer.TOKENID) @RequestParam("tokenId") Integer tokenId,
                                 @ApiParam(BtcWalletTransferApi.GetTransfer.WALLET_TYPE) @RequestParam(value = "walletType", defaultValue = BtcApplicationConstans.TYPE_CCT) String walletType) {
        String userOpenId = SSOHelper.getUserId(redisTemplate, request);
        //分页查询记录
        PageHelper.startPage(pageNum, pageSize);
        return ResultDTO.requstSuccess(btcWalletTransferService.selectTransfer(userOpenId, tokenId, walletType));
    }

    @ApiOperation(value = BtcWalletTransferApi.pcGetTransfer.METHOD_API_NAME, notes = BtcWalletTransferApi.pcGetTransfer.METHOD_API_NOTE)
    @GetMapping("/pcGetTransfer")
    public ResultDTO pcGetTransfer(HttpServletRequest request,
                                   @ApiParam(BtcWalletTransferApi.PAGENUM) @RequestParam(name = "pageNum", defaultValue = BaseConstant.PAGE_DEFAULT_INDEX) Integer pageNum,
                                   @ApiParam(BtcWalletTransferApi.PAGESIZE) @RequestParam(name = "pageSize", defaultValue = BaseConstant.PAGE_DEFAULT_SIZE) Integer pageSize,
                                   @ApiParam(BtcWalletTransferApi.pcGetTransfer.TOKENID) @RequestParam("tokenId") Integer tokenId,
                                   @ApiParam(BtcWalletTransferApi.pcGetTransfer.WALLET_TYPE) @RequestParam(value = "walletType", defaultValue = BtcApplicationConstans.TYPE_CCT) String walletType) {
        String userOpenId = SSOHelper.getUserId(redisTemplate, request);
        //分页查询记录
        PageHelper.startPage(pageNum, pageSize);
        List<BtcWalletTransferDTO> list = btcWalletTransferService.selectTransfer(userOpenId, tokenId, walletType);
        return generatePage(list);
    }

    @ApiOperation(value = BtcWalletTransferApi.Withdraw.METHOD_API_NAME, notes = BtcWalletTransferApi.Withdraw.METHOD_API_NOTE)
    @PostMapping("/withdraw")
    public ResultDTO withdraw(HttpServletRequest request,
                              @ApiParam(BtcWalletTransferApi.Withdraw.TOADDR) @RequestParam("toAddr") String toAddr,
                              @ApiParam(BtcWalletTransferApi.Withdraw.PASSWORD) @RequestParam("password") String password,
                              @ApiParam(BtcWalletTransferApi.Withdraw.AMOUNT) @RequestParam("amount") Double amount,
                              @ApiParam(BtcWalletTransferApi.Withdraw.TOKENID) @RequestParam("tokenId") Integer tokenId,
                              @ApiParam(BtcWalletTransferApi.Withdraw.WALLET_TYPE) @RequestParam(value = "walletType", defaultValue = BtcApplicationConstans.TYPE_CCT) String walletType) {
        String userOpenId = SSOHelper.getUserId(redisTemplate, request);
        return ResultDTO.requstSuccess(btcWalletTransferService.handleWithdraw(userOpenId, password, toAddr, tokenId, amount, walletType));
    }


}