package com.blockchain.server.btc.service;

import com.blockchain.server.btc.dto.BtcWalletDTO;
import com.blockchain.server.btc.entity.BtcWallet;
import com.blockchain.server.btc.entity.BtcWalletTransfer;

import java.util.Date;
import java.util.List;
import java.util.Set;

public interface BtcWalletService {

    /**
     * 创建托管账户
     *
     * @param userOpenId 系统用户ID
     * @return
     */
    Integer insertWallet(String userOpenId);

    /**
     * 获取用户某应用钱包的所有区块
     *
     * @param userOpenId 用户id
     * @param walletType 应用类型，币币交易等
     * @return
     */
    List<BtcWalletDTO> selectAllByUserOpenId(String userOpenId, String walletType);

    /**
     * 获取用户某应用钱包的所有区块
     *
     * @param userOpenId 用户id
     * @return
     */
    List<BtcWalletDTO> selectAllByUserOpenId(String userOpenId);

    /**
     * 获取用户钱包
     *
     * @param userOpenId 用户id
     * @param tokenId    币种id
     * @param walletType 应用类型，币币交易等
     * @return
     */
    BtcWalletDTO selectByUserOpenId(String userOpenId, Integer tokenId, String walletType);

    /**
     * 获取用户钱包，通过地址
     *
     * @param addr       用户id
     * @param tokenId    币种id
     * @param walletType 应用类型，币币交易等
     * @return
     */
    BtcWalletDTO selectByAddr(String addr, Integer tokenId, String walletType);

    /**
     * 加减钱包地址可用余额、冻结余额、总额
     *
     * @param address      地址
     * @param tokenId      币种id
     * @param freeAmount   可用余额加减数量
     * @param freezeAmount 冻结余额加减数量
     * @param totalAmount  总额加减数量
     * @param modifyTime   修改时间
     * @return
     */
    Integer updateBalanceByAddrInRowLock(String address, Integer tokenId, Double freeAmount, Double freezeAmount, Double totalAmount, Date modifyTime);

    /**
     * 获取所有用户托管钱包地址
     *
     * @return
     */
    Set<String> getAllWalletAddr();

    /**
     * 应用提现失败业务
     *
     * @param btcWalletTransfer 提现记录
     */
    void updateTxOutError(BtcWalletTransfer btcWalletTransfer);

    /**
     * 应用提现成功业务
     *
     * @param btcWalletTransfer 提现记录
     */
    void updateTxOutSuccess(BtcWalletTransfer btcWalletTransfer);

    /**
     * 根据用户ID,钱包类型，币种名称查询钱包信息
     *
     * @param userId
     * @param walletType
     * @param coinName
     * @return
     */
    BtcWallet findWallet(String userId, String walletType, String coinName);

    /**
     * 划转
     *
     * @param userId 用户ID
     * @param fromType 钱包类型
     * @param toType 钱包类型
     * @param coinName 币种名称
     * @param amount 币种金额
     * @return
     */
    BtcWalletTransfer handleTransfer(String userId, String fromType, String toType, String coinName, Double amount);

    int resetWallet(String userOpenId, String addr, String tokenId, String walletType);
}
