package com.blockchain.server.btc.service.impl;

import com.blockchain.server.btc.common.constants.BtcTransferConstans;
import com.blockchain.server.btc.common.enums.BtcEnums;
import com.blockchain.server.btc.common.exception.BtcException;
import com.blockchain.server.btc.dto.BtcApplicationDTO;
import com.blockchain.server.btc.dto.BtcTokenDTO;
import com.blockchain.server.btc.dto.BtcWalletDTO;
import com.blockchain.server.btc.entity.BtcToken;
import com.blockchain.server.btc.entity.BtcWallet;
import com.blockchain.server.btc.entity.BtcWalletTransfer;
import com.blockchain.server.btc.entity.WalletInitConfig;
import com.blockchain.server.btc.mapper.BtcWalletMapper;
import com.blockchain.server.btc.mapper.WalletInitConfigMapper;
import com.blockchain.server.btc.service.BtcApplicationService;
import com.blockchain.server.btc.service.BtcTokenService;
import com.blockchain.server.btc.service.BtcWalletService;
import com.blockchain.server.btc.service.BtcWalletTransferService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.UUID;

@Service
public class BtcWalletServiceImpl<psvm> implements BtcWalletService {

    @Autowired
    private BtcWalletTransferService btcWalletTransferService;

    @Autowired
    private BtcWalletMapper btcWalletMapper;

    @Autowired
    private BtcApplicationService btcApplicationService;

    @Autowired
    private BtcTokenService btcTokenService;
    @Autowired
    private WalletInitConfigMapper walletInitConfigMapper;

    @Override
    @Transactional
    public Integer insertWallet(String userOpenId) {
        List<BtcApplicationDTO> btcApplicationDTOList = btcApplicationService.listApplication();
        for (BtcApplicationDTO btcApplicationDTO : btcApplicationDTOList) {
            String address = "";
            try {
//                address = btcUtils.getNewAddress();
//                String privateKey = btcUtils.getPrivateKeyByAddress(address);
                //btc钱包地址长度为35位 uuid为36位
                address = UUID.randomUUID().toString().substring(1);
                String privateKey = "MEMlDf3GEMAKXxVi9XxJ6DCjJSKYoZXezMqeIWekcg455DJDCZhIPYbkiRKEy06T7iMaZVj3PPky+xmr7S32hwvRS2ANJ+OD/s5FODwmg5n2N3bhpLeRjqPvhFj3CCOXP01mBAodGUwJTDLy4UaZCOVr7dd9EAU33Xj37+OPcng=";
//                btcAddressSetRedisUtils.insert(address);
//                btcWalletKeyService.insertWalletKey(address, privateKey);
            } catch (Exception e) {
                throw new BtcException(BtcEnums.CREATE_WALLET_ERROR);
            }
            List<BtcTokenDTO> btcTokenDTOList = btcTokenService.listToken();
            for (BtcTokenDTO btcTokenDTO : btcTokenDTOList) {
                BtcWallet btcWallet = new BtcWallet();
                btcWallet.setAddr(address);
                btcWallet.setTokenId(btcTokenDTO.getTokenId());
                btcWallet.setUserOpenId(userOpenId);
                btcWallet.setTokenSymbol(btcTokenDTO.getTokenSymbol());
                btcWallet.setBalance(0.0);
                btcWallet.setFreeBalance(0.0);
                btcWallet.setFreezeBalance(0.0);
                btcWallet.setWalletType(btcApplicationDTO.getAppId());
                btcWallet.setCreateTime(new Date());
                btcWallet.setUpdateTime(btcWallet.getCreateTime());

                int count = btcWalletMapper.insertSelective(btcWallet);
                if (count != 1) {
                    throw new BtcException(BtcEnums.CREATE_WALLET_ERROR);
                }
            }
        }

        return 1;
    }

    @Override
    public List<BtcWalletDTO> selectAllByUserOpenId(String userOpenId, String walletType) {
        //校验钱包类型
        btcApplicationService.checkWalletType(walletType);
        return btcWalletMapper.selectAllByUserOpenId(userOpenId, walletType);
    }

    @Override
    public List<BtcWalletDTO> selectAllByUserOpenId(String userOpenId) {
        return btcWalletMapper.selectAll(userOpenId);
    }

    @Override
    public BtcWalletDTO selectByUserOpenId(String userOpenId, Integer tokenId, String walletType) {
        return btcWalletMapper.selectByUserOpenId(userOpenId, tokenId, walletType);
    }

    @Override
    public BtcWalletDTO selectByAddr(String addr, Integer tokenId, String walletType) {
        return btcWalletMapper.selectByAddr(addr, tokenId, walletType);
    }

    @Override
    public Integer updateBalanceByAddrInRowLock(String address, Integer tokenId, Double freeAmount, Double freezeAmount, Double totalAmount, Date modifyTime) {
        return btcWalletMapper.updateBalanceByAddrInRowLock(address, tokenId, freeAmount, freezeAmount, totalAmount, modifyTime);
    }

    @Override
    public Set<String> getAllWalletAddr() {
        return btcWalletMapper.getAllWalletAddr();
    }

    @Override
    @Transactional
    public void updateTxOutError(BtcWalletTransfer btcWalletTransfer) {
        BtcWalletDTO wallet = selectByAddr(btcWalletTransfer.getFromAddr(), btcWalletTransfer.getTokenId(), null);
        Date date = new Date();
        // 业务操作（1）- 恢复提现余额,扣除手续费
        Double amount = btcWalletTransfer.getAmount();
        this.updateBalanceByAddrInRowLock(wallet.getAddr(), wallet.getTokenId(), amount, amount * -1, Double.valueOf(0), date);
        // 业务操作（2）- 修改记录状态
        btcWalletTransferService.updateStatus(btcWalletTransfer.getId(), BtcTransferConstans.STATUS_FILE, date);
    }

    @Override
    @Transactional
    public void updateTxOutSuccess(BtcWalletTransfer btcWalletTransfer) {
        BtcWalletDTO wallet = selectByAddr(btcWalletTransfer.getFromAddr(), btcWalletTransfer.getTokenId(), null);
        Date date = new Date();
        // 业务操作（1）- 恢复提现余额,扣除手续费
        Double amount = btcWalletTransfer.getAmount();
        this.updateBalanceByAddrInRowLock(wallet.getAddr(), wallet.getTokenId(), Double.valueOf(0), amount * -1, amount * -1, date);
        // 业务操作（2）- 修改记录状态
        btcWalletTransferService.updateStatus(btcWalletTransfer.getId(), BtcTransferConstans.STATUS_SUCCESS, date);
    }

    @Override
    public BtcWallet findWallet(String userId, String walletType, String coinName) {
        BtcWallet wallet = new BtcWallet();
        wallet.setWalletType(walletType);
        wallet.setTokenSymbol(coinName);
        wallet.setUserOpenId(userId);
        BtcWallet btcWallet = btcWalletMapper.selectOne(wallet);
        if (btcWallet == null) {
            throw new BtcException(BtcEnums.INEXISTENCE_WALLET);
        }
        return btcWallet;
    }

    @Override
    @Transactional
    public BtcWalletTransfer handleTransfer(String userId, String fromType, String toType, String coinName, Double amount) {
        Date end = new Date();
        amount = new BigDecimal(amount.toString()).abs().doubleValue();
        //校验钱包类型
        btcApplicationService.checkWalletType(fromType);
        btcApplicationService.checkWalletType(toType);
        BtcToken btcToken = btcTokenService.selectByTokenName(coinName);
        BtcWallet fromWallet = this.findWallet(userId, fromType, coinName);
        BtcWallet toWallet = this.findWallet(userId, toType, coinName);
        //该用户减去提现可用余额、总额
        int countUb = this.updateBalanceByAddrInRowLock(fromWallet.getAddr(), fromWallet.getTokenId(), -amount, 0.0, -amount, end);
        if (countUb != 1) {
            throw new BtcException(BtcEnums.WITHDRAW_ERROR);
        }
        //接收用户加上可用余额、总额
        int countUbR = this.updateBalanceByAddrInRowLock(toWallet.getAddr(), toWallet.getTokenId(), amount, 0.0, amount, end);
        if (countUbR != 1) {
            throw new BtcException(BtcEnums.WITHDRAW_ERROR);
        }

        //并插入一条提现记录，站内快速转账
        BtcWalletTransfer btcWalletTransfer = new BtcWalletTransfer();
        btcWalletTransfer.setId(UUID.randomUUID().toString());
        btcWalletTransfer.setFromAddr(fromWallet.getAddr());
        btcWalletTransfer.setToAddr(toWallet.getAddr());
        btcWalletTransfer.setAmount(Math.abs(amount));
        btcWalletTransfer.setTokenId(btcToken.getTokenId());
        btcWalletTransfer.setTokenSymbol(btcToken.getTokenSymbol());
        btcWalletTransfer.setTransferType(BtcTransferConstans.TYPE_FAST);
        btcWalletTransfer.setStatus(BtcTransferConstans.STATUS_SUCCESS);
        btcWalletTransfer.setCreateTime(end);
        btcWalletTransfer.setUpdateTime(end);
        int countIt = btcWalletTransferService.insertTransfer(btcWalletTransfer);
        if (countIt != 1) {
            throw new BtcException(BtcEnums.WITHDRAW_ERROR);
        }
        return btcWalletTransfer;
    }

    @Override
    public int resetWallet(String userOpenId, String addr, String tokenId, String walletType) {

        WalletInitConfig walletInitConfig = new WalletInitConfig();
        walletInitConfig.setCoin(tokenId);
        walletInitConfig.setWalletType(walletType);
        WalletInitConfig config = walletInitConfigMapper.selectOne(walletInitConfig);
        if (config == null || config.getBalance() == null) {
            throw new BtcException(BtcEnums.RESET_ERROR);
        }
        btcWalletMapper.updateWalletBalanceByUserOpenId(config.getBalance(), userOpenId, tokenId, walletType, new Date());

        return 1;
    }

}
