package com.blockchain.server.cct.inner;

import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.server.cct.dto.order.PublishOrderDTO;
import com.blockchain.server.cct.service.MatchService;
import com.blockchain.server.cct.service.PublishOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author huangxl
 * @create 2019-04-25 19:02
 */
@RestController
@RequestMapping("/inner")
public class CctInner {
    @Autowired
    private MatchService matchService;
    @Autowired
    private PublishOrderService publishOrderService;


    @GetMapping("/send")
    public ResultDTO send(@RequestParam("coinName") String coinName,
                          @RequestParam("unitName") String unitName) {
        matchService.convertAndSend(coinName, unitName);
        return ResultDTO.requstSuccess();
    }

    /***
     * 查询单价范围内的挂单
     * @param coinName
     * @param unitName
     * @param minPrice
     * @param maxPrice
     * @return
     */
    @PostMapping("/listBeMatchOrderToBot")
    public ResultDTO listBeMatchOrderToBot(@RequestParam("coinName") String coinName,
                                           @RequestParam("unitName") String unitName,
                                           @RequestParam("minPrice") BigDecimal minPrice,
                                           @RequestParam("maxPrice") BigDecimal maxPrice) {
        List<PublishOrderDTO> orderDTOS = matchService.listBeMatchOrderToBot(coinName, unitName, minPrice, maxPrice);
        return ResultDTO.requstSuccess(orderDTOS);
    }

    /****
     * 发布限价买单 - 用于撮合机器人
     * @param userId
     * @param coinName
     * @param unitName
     * @param totalNum
     * @param price
     * @return
     */
    @PostMapping("/handleLimitBuyToBot")
    public ResultDTO handleLimitBuyToBot(@RequestParam("userId") String userId,
                                         @RequestParam("coinName") String coinName,
                                         @RequestParam("unitName") String unitName,
                                         @RequestParam("totalNum") BigDecimal totalNum,
                                         @RequestParam("price") BigDecimal price) {
        String pubilshId = publishOrderService.handleLimitBuyToBot(userId, coinName, unitName, totalNum, price);
        return ResultDTO.requstSuccess(pubilshId);
    }

    /***
     * 发布限价卖单 - 用于撮合机器人
     * @param userId
     * @param coinName
     * @param unitName
     * @param totalNum
     * @param price
     * @return
     */
    @PostMapping("/handleLimitSellToBot")
    public ResultDTO handleLimitSellToBot(@RequestParam("userId") String userId,
                                          @RequestParam("coinName") String coinName,
                                          @RequestParam("unitName") String unitName,
                                          @RequestParam("totalNum") BigDecimal totalNum,
                                          @RequestParam("price") BigDecimal price) {
        String pubilshId = publishOrderService.handleLimitSellToBot(userId, coinName, unitName, totalNum, price);
        return ResultDTO.requstSuccess(pubilshId);
    }

    /***
     * 撮合 - 用于撮合机器人
     * @param matchId
     * @param bymatchId
     * @return
     */
    @PostMapping("/handleMatchToBot")
    public ResultDTO handleMatchToBot(@RequestParam("matchId") String matchId,
                                      @RequestParam("bymatchId") String bymatchId) {
        Boolean flag = matchService.handleMatch(matchId, bymatchId);
        return ResultDTO.requstSuccess(flag);
    }
}
