package com.blockchain.server.cct.service.impl;

import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.server.cct.dto.matchconfig.MatchConfigDTO;
import com.blockchain.server.cct.dto.rpc.currency.CurrencyMarketDTO;
import com.blockchain.server.cct.entity.MatchConfig;
import com.blockchain.server.cct.entity.TradingRecord;
import com.blockchain.server.cct.feign.CurrencyFeign;
import com.blockchain.server.cct.mapper.MatchConfigMapper;
import com.blockchain.server.cct.service.MatchConfigService;
import com.blockchain.server.cct.service.TradingRecordService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;

@Service
public class MatchConfigServiceImpl implements MatchConfigService {

    @Autowired
    private MatchConfigMapper matchConfigMapper;
    @Autowired
    private CurrencyFeign currencyFeign;


    @Override
    public MatchConfigDTO getByCoinAndUnitBeforeToday(String coinName, String unitName) {
        MatchConfig matchConfig = matchConfigMapper.selectByCoinAndUnitBeforeToday(coinName, unitName);
        CurrencyMarketDTO currency = getCurrency(coinName, unitName);
        //判空
        if (matchConfig != null) {
            MatchConfigDTO matchConfigDTO = new MatchConfigDTO();
            BeanUtils.copyProperties(matchConfig, matchConfigDTO);

            //最新行情判空
            if (currency != null) {
                //挂单金额作为最新成交价
                BigDecimal price = currency.getAmount();
                //最大价格 = 最新成交价 + （最新成交价 * 最大涨幅）
                matchConfigDTO.setMaxPrice(price.add(price.multiply(matchConfigDTO.getMaxPercent())));
                //最小价格 = 最新成交价 - （最新成交价 * 最小跌幅）
                matchConfigDTO.setMinPrice(price.subtract(price.multiply(matchConfigDTO.getMinPercent())));
            }
            return matchConfigDTO;
        }

        return null;
    }

    /***
     * 获取最新行情
     * @param coinName
     * @param unitName
     * @return
     */
    private CurrencyMarketDTO getCurrency(String coinName, String unitName) {
        String currencyPari = coinName + "-" + unitName;
        ResultDTO<CurrencyMarketDTO> resultDTO = currencyFeign.getLast(currencyPari);
        return resultDTO.getData();
    }
}
