package com.blockchain.server.currency.Scheduling;

import com.alibaba.fastjson.JSONObject;
import com.blockchain.common.base.util.HttpUtilManager;
import com.blockchain.server.currency.dto.CurrencyMarketDTO;
import com.blockchain.server.currency.service.CurrencyMarketService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
@Configurable
@EnableScheduling
public class HuobiMarketScheduling {

    @Autowired
    private CurrencyMarketService currencyMarketService;

    @Value("${market.huobi.url}")
    private String huobiUrl;

    @Value("${market.huobi.currencys}")
    private String huobiCurrencys;

    private static ExecutorService cachedThreadPool = Executors.newSingleThreadExecutor();

    @Scheduled(cron = "*/1 * * * * ?")
    public void getCurrencyMarket() {
        cachedThreadPool.execute(new Runnable() {
            @Override
            public void run() {
                setHuobiMarket();
            }
        });
    }

    private void setHuobiMarket() {
        List<List> currencys = JSONObject.parseArray(huobiCurrencys,List.class);
        for (List data : currencys){
            saveMarket(getHuobiMarket(huobiUrl + data.get(0).toString()),
                    data.get(1).toString(),data.get(2).toString());
        }
    }

    /**
     *
     * @param list
     * @param coinName
     * @param unitName
     */
    private void saveMarket(List<JSONObject> list, String coinName, String unitName) {
        CurrencyMarketDTO now = currencyMarketService.get(coinName + "-" + unitName);
        for (JSONObject obj : list) {
            Long timestamp = Long.parseLong(obj.get("ts").toString());
            if(timestamp >= now.getTimestamp()) {
                List<JSONObject> data = (List<JSONObject>)obj.get("data");
                currencyMarketService.save(coinName, unitName,
                        new BigDecimal(data.get(0).get("price").toString()),
                        new BigDecimal(data.get(0).get("amount").toString()),
                        timestamp,data.get(0).get("direction").toString().toUpperCase());
            } else {
                break;
            }
        }
    }


    /**
     * coinbase上的美元行情
     *
     * @return
     */
    private List<JSONObject> getHuobiMarket(String url) {
        HttpUtilManager manager = HttpUtilManager.getInstance();
        String result = manager.requestHttpGet(url);
        JSONObject jsonObject = JSONObject.parseObject(result);
        return (List<JSONObject>) jsonObject.get("data");
    }


}
