package com.blockchain.server.eos.service.impl;

import com.blockchain.common.base.util.ExceptionPreconditionUtils;
import com.blockchain.server.eos.dto.WalletDTO;
import com.blockchain.server.eos.dto.WalletTransferDTO;
import com.blockchain.server.eos.entity.WalletTransfer;
import com.blockchain.server.eos.mapper.WalletTransferMapper;
import com.blockchain.server.eos.service.EosWalletService;
import com.blockchain.server.eos.service.EosWalletTransferService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author Harvey
 * @date 2019/2/19 11:40
 * @user WIN10
 */
@Service
public class EosWalletTransferServiceImpl implements EosWalletTransferService {

    @Autowired
    private EosWalletService eosWalletService;
    @Autowired
    private WalletTransferMapper walletTransferMapper;

    /**
     * 插入交易记录
     *
     * @param walletTransfer
     * @return
     */
    @Override
    @Transactional
    public int insertWalletTransfer(WalletTransfer walletTransfer) {
        ExceptionPreconditionUtils.notEmpty(walletTransfer);
        return walletTransferMapper.insertSelective(walletTransfer);
    }

    /**
     * 查询钱包交易记录
     *
     * @param userOpenId
     * @param tokenName
     * @param walletType
     * @return
     */
    @Override
    public List<WalletTransferDTO> listWalletTransfer(String userOpenId, String tokenName, String walletType) {
        WalletDTO walletDTO = eosWalletService.selectWallet(userOpenId, tokenName, walletType);
        Integer walletId = walletDTO.getId();
        return walletTransferMapper.listWalletTransfer(walletId, tokenName);
    }

}
