package com.blockchain.server.eth.controller;

import com.blockchain.common.base.constant.BaseConstant;
import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.common.base.dto.SessionUserDTO;
import com.blockchain.common.base.exception.RPCException;
import com.blockchain.common.base.util.SSOHelper;
import com.blockchain.server.eth.common.enums.EthWalletEnums;
import com.blockchain.server.eth.common.exception.EthWalletException;
import com.blockchain.server.eth.controller.api.EthTokenApi;
import com.blockchain.server.eth.controller.api.EthWalletApi;
import com.blockchain.server.eth.entity.EthWalletTransfer;
import com.blockchain.server.eth.feign.UserFeign;
import com.blockchain.server.eth.service.IEthTokenService;
import com.blockchain.server.eth.service.IEthWalletKeyService;
import com.blockchain.server.eth.service.IEthWalletService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.tomcat.jni.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;

/**
 * @author YH
 * @date 2018年11月6日15:51:56
 */
@Api(EthWalletApi.MARKET_CONTROLLER_API)
@RestController
@RequestMapping("/wallet")
public class EthWalletController {
    @Autowired
    IEthWalletService ethWalletService;
    @Autowired
    IEthWalletKeyService ethWalletKeyService;
    @Autowired
    RedisTemplate redisTemplate;
    @Autowired
    private UserFeign userFeign;


    @ApiOperation(value = EthWalletApi.GetWallet.METHOD_API_NAME, notes = EthWalletApi.GetWallet.METHOD_API_NOTE)
    @GetMapping("/getWallet")
    public ResultDTO getWallet(
            @ApiParam(EthWalletApi.GetWallet.METHOD_API_WALLETTYPE) @RequestParam(name = "walletType", required =
                    false) String walletType,
            @ApiParam(EthWalletApi.GetWallet.METHOD_API_TOKENADDR) @RequestParam(name = "tokenAddr",
                    required = false) String tokenAddr,
            HttpServletRequest request) {
        String userId = SSOHelper.getUserId(redisTemplate, request);
        return ResultDTO.requstSuccess(ethWalletService.selectByUserOpenIdAndTokenAddrAndWalletType(userId, tokenAddr
                , walletType));
    }

    @ApiOperation(value = EthWalletApi.GetWallets.METHOD_API_NAME, notes = EthWalletApi.GetWallets.METHOD_API_NOTE)
    @GetMapping("/getWallets")
    public ResultDTO getWallets(
            @ApiParam(EthWalletApi.GetWallets.METHOD_API_WALLETTYPE) @RequestParam(name = "walletType", required =
                    false) String walletType,
            HttpServletRequest request) {
        String userId = SSOHelper.getUserId(redisTemplate, request);
        return ResultDTO.requstSuccess(ethWalletService.selectByUserOpenIdAndWalletType(userId, walletType));
    }

    @ApiOperation(value = EthWalletApi.GetPublicKey.METHOD_API_NAME, notes = EthWalletApi.GetPublicKey.METHOD_API_NOTE)
    @GetMapping("/getPublicKey")
    public ResultDTO getPublicKey(HttpServletRequest request) {
        String userId = SSOHelper.getUserId(redisTemplate, request);
        return ResultDTO.requstSuccess(ethWalletService.getPublicKey(userId));
    }

    @ApiOperation(value = EthWalletApi.CreationWallet.METHOD_API_NAME, notes =
            EthWalletApi.CreationWallet.METHOD_API_NOTE)
    @PostMapping("/creationWallet")
    public ResultDTO creationWallet(@ApiParam(EthWalletApi.CreationWallet.METHOD_API_WALLETTYPE) @RequestParam(name =
            "walletType", required = false) String walletType,
                                    @ApiParam(EthWalletApi.CreationWallet.METHOD_API_TOKENADDR) @RequestParam(name =
                                            "tokenAddr", required = false) String tokenAddr,
                                    @ApiParam(EthWalletApi.CreationWallet.METHOD_API_PASSWORD) @RequestParam(name =
                                            "password", required = false) String password,
                                    HttpServletRequest request) {
        String userId = SSOHelper.getUserId(redisTemplate, request);
        return ResultDTO.requstSuccess(ethWalletService.insert(userId, tokenAddr, walletType, password));
    }

    @ApiOperation(value = EthWalletApi.SaveWalletPass.METHOD_API_NAME, notes =
            EthWalletApi.SaveWalletPass.METHOD_API_NOTE)
    @PostMapping("/saveWalletPass")
    public ResultDTO saveWalletPass(
            @ApiParam(EthWalletApi.SaveWalletPass.METHOD_API_PASSWORD) @RequestParam(name = "password", required = false) String password,
            @ApiParam(EthWalletApi.SaveWalletPass.METHOD_API_CODE) @RequestParam(name = "code", required = false) String code,
            HttpServletRequest request) {
        SessionUserDTO user = SSOHelper.getUser(redisTemplate, request);
        ResultDTO resultDTO = userFeign.validateSmsg(code, user.getTel());
        if (resultDTO == null) {
            throw new EthWalletException(EthWalletEnums.SERVER_IS_TOO_BUSY);
        }
        if (resultDTO.getCode() != BaseConstant.REQUEST_SUCCESS) {
            throw new RPCException(resultDTO.getCode(), resultDTO.getMsg());
        }
        ethWalletService.updatePassword(user.getId(), password);
        return ResultDTO.requstSuccess();
    }

    @ApiOperation(value = EthWalletApi.ExistsPass.METHOD_API_NAME, notes = EthWalletApi.ExistsPass.METHOD_API_NOTE)
    @GetMapping("/existsPass")
    public ResultDTO existsPass(HttpServletRequest request) {
        String userId = SSOHelper.getUserId(redisTemplate, request);
        boolean existsPass = ethWalletKeyService.existsPass(userId);
        return ResultDTO.requstSuccess(existsPass);
    }

    @ApiOperation(value = EthWalletApi.WithdrawDeposit.METHOD_API_NAME, notes =
            EthWalletApi.WithdrawDeposit.METHOD_API_NOTE)
    @PostMapping("/withdrawDeposit")
    public ResultDTO withdrawDeposit(
            @ApiParam(EthWalletApi.WithdrawDeposit.METHOD_API_WALLETTYPE) @RequestParam(name = "walletType",
                    required = false) String walletType,
            @ApiParam(EthWalletApi.WithdrawDeposit.METHOD_API_TOKENADDR) @RequestParam(name = "tokenAddr", required =
                    false) String tokenAddr,
            @ApiParam(EthWalletApi.WithdrawDeposit.METHOD_API_PASSWORD) @RequestParam(name = "password", required =
                    false) String password,
            @ApiParam(EthWalletApi.WithdrawDeposit.METHOD_API_TOADDDR) @RequestParam(name = "toAddr", required =
                    false) String toAddr,
            @ApiParam(EthWalletApi.WithdrawDeposit.METHOD_API_AMOUNT) @RequestParam(name = "amount",
                    required = false) String amount,
            HttpServletRequest request) {
        String userId = SSOHelper.getUserId(redisTemplate, request);
        EthWalletTransfer tx = ethWalletService.handleWelletOutApply(userId, tokenAddr, toAddr, walletType, amount,
                password);
        return ResultDTO.requstSuccess(tx);
    }

    @ApiOperation(value = EthWalletApi.Transfer.METHOD_API_NAME, notes = EthWalletApi.Transfer.METHOD_API_NOTE)
    @PostMapping("/transfer")
    public ResultDTO handleTransfer(HttpServletRequest request,
                                    @ApiParam(EthWalletApi.Transfer.METHOD_API_FROMTYPE) @RequestParam("fromType") String fromType,
                                    @ApiParam(EthWalletApi.Transfer.METHOD_API_TOTYPE) @RequestParam("toType") String toType,
                                    @ApiParam(EthWalletApi.Transfer.METHOD_API_COINNAME) @RequestParam("coinName") String coinName,
                                    @ApiParam(EthWalletApi.Transfer.METHOD_API_AMOUNT) @RequestParam("amount") BigDecimal amount) {
        String userOpenId = SSOHelper.getUserId(redisTemplate, request);
        return ResultDTO.requstSuccess(ethWalletService.handleTransfer(userOpenId,fromType, toType, coinName, amount));
    }

    @ApiOperation(value = EthWalletApi.ResetWallet.METHOD_API_NAME, notes = EthWalletApi.ResetWallet.METHOD_API_NOTE)
    @PostMapping("/resetWallet")
    public ResultDTO resetWallet(HttpServletRequest request,
                                 @ApiParam(EthWalletApi.ResetWallet.TOADDR) @RequestParam("addr") String addr,
                                 @ApiParam(EthWalletApi.ResetWallet.TOKENID) @RequestParam("tokenId") String tokenId,
                                 @ApiParam(EthWalletApi.ResetWallet.WALLET_TYPE) @RequestParam(value = "walletType") String walletType) {
        String userOpenId = SSOHelper.getUserId(redisTemplate, request);
        ethWalletService.resetWallet(userOpenId, addr, tokenId, walletType);
        return ResultDTO.requstSuccess();
    }

}
