package com.blockchain.server.eth.inner;

import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.common.base.util.SSOHelper;
import com.blockchain.server.eth.dto.EthWalletDTO;
import com.blockchain.server.eth.inner.api.EthWalletApi;
import com.blockchain.server.eth.service.IEthWalletService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author YH
 * @date 2018年11月6日15:51:56
 */
@Api(EthWalletApi.MARKET_CONTROLLER_API)
@RestController
@RequestMapping("/inner/wallet")
public class EthWalletInner {
    private static final Logger LOG = LoggerFactory.getLogger(EthWalletInner.class);
    @Autowired
    IEthWalletService ethWalletService;
    @Autowired
    RedisTemplate redisTemplate;

    @ApiOperation(value = EthWalletApi.IsPassword.METHOD_API_NAME, notes = EthWalletApi.IsPassword.METHOD_API_NOTE)
    @GetMapping("/isPassword")
    public ResultDTO isPassword(@ApiParam(EthWalletApi.IsPassword.METHOD_API_PASSWORD) @RequestParam(name = "password", required = false) String password,
                                HttpServletRequest request) {
        String userId = SSOHelper.getUserId(redisTemplate, request);
        LOG.info("校验密码，密码：" + password);
        ethWalletService.isPassword(userId, password);
        return ResultDTO.requstSuccess();
    }

    @ApiOperation(value = EthWalletApi.GetPublicKey.METHOD_API_NAME, notes = EthWalletApi.GetPublicKey.METHOD_API_NOTE)
    @GetMapping("/getPublicKey")
    public ResultDTO getPublicKey(@ApiParam(EthWalletApi.GetPublicKey.METHOD_API_USERID) @RequestParam("userOpenId") String userOpenId, HttpServletRequest request) {
        LOG.info("获取公钥密码，userOpenId：" + userOpenId);
        return ResultDTO.requstSuccess(ethWalletService.getPublicKey(userOpenId));
    }

    @ApiOperation(value = EthWalletApi.InitWallets.METHOD_API_NAME, notes = EthWalletApi.InitWallets.METHOD_API_NOTE)
    @GetMapping("/initWallets")
    public ResultDTO initWallets(@ApiParam(EthWalletApi.InitWallets.METHOD_API_USERID) @RequestParam("userOpenId") String userOpenId) {
        LOG.info("获取公钥密码，userOpenId：" + userOpenId);
        ethWalletService.insertInit(userOpenId);
        return ResultDTO.requstSuccess();
    }

    @GetMapping("/getWallets")
    public ResultDTO<List<EthWalletDTO>> getWallets(@RequestParam("userId") String userId) {
        return ResultDTO.requstSuccess(ethWalletService.selectByUserOpenId(userId));
    }

}
