package com.blockchain.server.user.common.utils;

import com.blockchain.common.base.constant.BaseConstant;
import com.blockchain.server.user.common.constants.other.RedisConstant;
import com.blockchain.server.user.common.enums.UserEnums;
import com.blockchain.server.user.common.exceprion.UserException;
import lombok.Data;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.mail.Message;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

/**
 * @author huangxl
 * @create 2019-01-18 18:05
 */
@Data
@Component
@ConfigurationProperties(prefix = "email")
public class EMailTransmitHelper {
    private static final Logger LOG = LoggerFactory.getLogger(EMailTransmitHelper.class);
    private String fromAccount;//账号
    private String hostName;//smtp服务器
    private String userName;//发件人姓名
    private String authCode;//授权码、密码
    private String port;//端口号
    private int timeout = 30;//超时时间，分钟
    private boolean closed;
    @Autowired
    private RedisTemplate redisTemplate;

    //邮箱验证码 一般发送
    @Async
    public void sendEmail(String emailaddress, String code, String userLocale,String redisKey) {
        if (closed) {
            //判断是否关闭了邮箱发送功能，如果是关闭状态，直接返回结果
            return;
        }
        Properties props = new Properties();
        props.put("mail.smtp.host", hostName);
        props.put("mail.smtp.socketFactory.port", port);
        props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
        props.put("mail.smtp.auth", "true");
        props.put("mail.smtp.port", port);
        Session session = Session.getDefaultInstance(props,
                new javax.mail.Authenticator() {
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(fromAccount, authCode);
                    }
                });
        try {
            String sendUserName = MimeUtility.encodeText(userName);//发件人姓名
            Message message = new MimeMessage(session);
            message.setFrom(new InternetAddress(fromAccount,sendUserName));
            message.setRecipients(Message.RecipientType.TO,
                    InternetAddress.parse(emailaddress));
            message.setSubject(Subject.getSubject(userLocale));
            message.setText(MessageFormat.format(Msg.getSubject(userLocale), code));
            Transport.send(message);
        } catch (Exception e) {
            LOG.error("发送邮件失败，邮箱地址为：" + emailaddress);
            redisTemplate.delete(redisKey);//移除缓存的key
            e.printStackTrace();
            throw new UserException(UserEnums.SEND_CODE_ERROR);
        }
    }

    /**
     * 邮箱主题
     */
    private enum Subject {
        ZH_CN("【ZHIXINCHAIN】安全验证"),
        ZH_HK("【ZHIXINCHAIN】安全驗證"),
        EN_US("【ZHIXINCHAIN】Safety Verification");
        private static Map<String, Subject> map = new HashMap<>();

        static {
            map.put(BaseConstant.USER_LOCALE_ZH_HK, ZH_HK);
            map.put(BaseConstant.USER_LOCALE_ZH_CN, ZH_CN);
            map.put(BaseConstant.USER_LOCALE_EN_US, EN_US);
        }

        private String value;

        Subject(String value) {
            this.value = value;
        }

        public static String getSubject(String userLocale) {
            return map.getOrDefault(userLocale, map.get(BaseConstant.USER_LOCALE_DEFAULT)).value;
        }

    }

    /**
     * 邮箱主题
     */
    private enum Msg {
        ZH_CN("您好: \n【ZHIXINCHAIN】本次安全验证为:  {0} ,验证码仅用于智信链官网，如非本人操作，请忽略！"),
        ZH_HK("您好: \n【ZHIXINCHAIN】本次安全驗證為: {0} ,驗證碼僅用於智信鏈官網，如非本人操作，請忽略！"),
        EN_US("Dear: \n【ZHIXINCHAIN】your verification code is: {0} ,The verification code is only used for ZHIXINCHAIN official website. If you do not operate the code, please ignore it!");
        private static Map<String, Msg> map = new HashMap<>();

        static {
            map.put(BaseConstant.USER_LOCALE_ZH_HK, ZH_HK);
            map.put(BaseConstant.USER_LOCALE_ZH_CN, ZH_CN);
            map.put(BaseConstant.USER_LOCALE_EN_US, EN_US);
        }

        private String value;

        Msg(String value) {
            this.value = value;
        }

        public static String getSubject(String userLocale) {
            return map.getOrDefault(userLocale, map.get(BaseConstant.USER_LOCALE_DEFAULT)).value;
        }

    }

}
