package com.blockchain.server.user.service.impl;

import com.blockchain.common.base.util.ExceptionPreconditionUtils;
import com.blockchain.server.user.common.constants.sql.AuthenticationApplyConstant;
import com.blockchain.server.user.common.constants.sql.UserAuthenticationConstant;
import com.blockchain.server.user.common.constants.sql.UserInfoConstant;
import com.blockchain.server.user.common.enums.UserEnums;
import com.blockchain.server.user.common.exceprion.UserException;
import com.blockchain.server.user.common.utils.FileUploadHelper;
import com.blockchain.server.user.entity.*;
import com.blockchain.server.user.mapper.AuthenticationApplyMapper;
import com.blockchain.server.user.mapper.HighAuthenticationApplyMapper;
import com.blockchain.server.user.mapper.UserAuthenticationMapper;
import com.blockchain.server.user.service.AuthenticationApplyLogService;
import com.blockchain.server.user.service.AuthenticationApplyService;
import com.blockchain.server.user.service.UserInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.UUID;

/**
 * @author huangxl
 * @create 2019-02-26 14:26
 */
@Service
public class AuthenticationApplyServiceImpl implements AuthenticationApplyService {
    @Autowired
    private UserInfoService userInfoService;
    @Autowired
    private AuthenticationApplyMapper authenticationApplyMapper;
    @Autowired
    private HighAuthenticationApplyMapper highAuthenticationApplyMapper;
    @Autowired
    private AuthenticationApplyLogService authenticationApplyLogService;
    @Autowired
    private UserAuthenticationMapper userAuthenticationMapper;


    @Override
    public String selectHighRemarkByUserId(String userId) {
        AuthenticationApplyLog authenticationApplyLog = authenticationApplyLogService.selectRemarkByUserId(UserAuthenticationConstant.HIGH,userId);
        return authenticationApplyLog!=null?authenticationApplyLog.getRemark():null;
    }

    @Override
    public String selectLowRemarkByUserId(String userId) {
        AuthenticationApplyLog authenticationApplyLog = authenticationApplyLogService.selectRemarkByUserId(UserAuthenticationConstant.LOW,userId);
        return authenticationApplyLog!=null?authenticationApplyLog.getRemark():null;
    }

    @Override
    @Transactional
    public void insertBasicAuth(String userId, String realName, String idNumber, String type, String imgs) {
        ExceptionPreconditionUtils.notEmpty(userId, realName, idNumber, imgs);
        FileUploadHelper.verifyFileNameList(imgs, 2);//初级认证需要身份证正反面，手持身份证照片
        if (StringUtils.isEmpty(type)) {
            type = AuthenticationApplyConstant.TYPE_DEFAULT;
        }

        UserInfo userInfo = userInfoService.selectByUserId(userId);
        if (userInfo.getLowAuth().equals(UserInfoConstant.STATUS_LOW_AUTH_WAIT)) {
            //已经是等待审核状态
            throw new UserException(UserEnums.AUTH_WAIT);
        }
        if (userInfo.getLowAuth().equals(UserInfoConstant.STATUS_LOW_AUTH_YES)) {
            //已经是认证状态
            throw new UserException(UserEnums.AUTH_YES);
        }

        Date now = new Date();
        String[] filePath = imgs.split(",");


        UserAuthentication apply = new UserAuthentication();
        apply.setId(UUID.randomUUID().toString());
        apply.setUserId(userId);
        apply.setCreateTime(now);
        apply.setFileUrl1(filePath[0]);
        apply.setFileUrl2(filePath[1]);
        // apply.setFileUrl3(filePath[2]);
        apply.setIdNumber(idNumber);
        apply.setRealName(realName);
        apply.setType(type);
        apply.setModifyTime(now);
        userAuthenticationMapper.insert(apply);

        userInfo.setLowAuth(UserInfoConstant.STATUS_LOW_AUTH_YES);
        userInfo.setModifyTime(now);
        userInfoService.updateUserInfo(userInfo);
    }

    @Override
    @Transactional
    public void insertHighAuth(String userId, String img) {
        ExceptionPreconditionUtils.notEmpty(userId, img);
        FileUploadHelper.verifyFileName(img);
        UserInfo userInfo = userInfoService.selectByUserId(userId);
        if (!userInfo.getLowAuth().equals(UserInfoConstant.STATUS_LOW_AUTH_YES)) {
            //提交高级认证前必须经过初级认证
            throw new UserException(UserEnums.AUTH_BASIC_BEFORE);
        }
        if (userInfo.getHighAuth().equals(UserInfoConstant.STATUS_HIGHT_AUTH_WAIT)) {
            //已经是等待审核状态
            throw new UserException(UserEnums.AUTH_WAIT);
        }
        if (userInfo.getHighAuth().equals(UserInfoConstant.STATUS_HIGHT_AUTH_YES)) {
            //已经是认证状态
            throw new UserException(UserEnums.AUTH_YES);
        }
        Date now = new Date();

//        HighAuthenticationApply apply = new HighAuthenticationApply();
//        apply.setId(UUID.randomUUID().toString());
//        apply.setCreateTime(now);
//        apply.setFileUrl(img);
//        apply.setModifyTime(now);
//        apply.setStatus(AuthenticationApplyConstant.STATUS_WAIT);
//        apply.setUserId(userId);
//
//        highAuthenticationApplyMapper.insert(apply);

        UserAuthentication exist = new UserAuthentication();
        exist.setUserId(userId);
        UserAuthentication userAuthentication = userAuthenticationMapper.selectOne(exist);
        userAuthentication.setFileUrl3(img);
        userAuthentication.setModifyTime(now);
        userAuthenticationMapper.updateByPrimaryKey(userAuthentication);

        userInfo.setHighAuth(UserInfoConstant.STATUS_HIGHT_AUTH_YES);
        userInfo.setModifyTime(now);
        userInfoService.updateUserInfo(userInfo);
    }

    /**
     * 判断用户认证状态接口
     * @param userId
     * @param authenticationType
     * @return
     */
    @Override
    public String judgeAuthentication(String userId, String authenticationType) {
        if (UserAuthenticationConstant.LOW.equalsIgnoreCase(authenticationType))
            return authenticationApplyMapper.judgeAuthentication(userId);
        else if (UserAuthenticationConstant.HIGH.equalsIgnoreCase(authenticationType))
            return highAuthenticationApplyMapper.judgeAuthentication(userId);
        return null;
    }
}
