package com.blockchain.server.user.service.impl;

import com.blockchain.common.base.util.ExceptionPreconditionUtils;
import com.blockchain.server.user.common.enums.UserEnums;
import com.blockchain.server.user.common.exceprion.UserException;
import com.blockchain.server.user.common.utils.CommonUtils;
import com.blockchain.server.user.entity.UserInfo;
import com.blockchain.server.user.entity.UserRelation;
import com.blockchain.server.user.mapper.UserRelationMapper;
import com.blockchain.server.user.service.UserInfoService;
import com.blockchain.server.user.service.UserRelationService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.UUID;

@Service
public class UserRelationServiceImpl implements UserRelationService {

    @Autowired
    private UserRelationMapper userRelationMapper;
    @Autowired
    private UserInfoService userInfoService;

    @Override
    @Transactional
    public int insertRelationChain(String userId, String invitationCode) {
        ExceptionPreconditionUtils.notEmpty(userId);
        UserRelation userRelation = new UserRelation();
        //如果邀请码不为空，插入用户关系链
        if (StringUtils.isEmpty(invitationCode)) {
            userRelation.setPid(null);
            userRelation.setRelationChain(userId);
            userRelation.setTreeDepth(0);
        } else {
            Integer incrCode = CommonUtils.getUserIncrCodeFromInvitationCode(invitationCode);//解析出用户自增长码
            if (incrCode == null) {
                throw new UserException(UserEnums.INVALID_INVITATION_CODE);
            }
            UserInfo userInfo = userInfoService.selectByIncrCode(incrCode);
            if (userInfo == null) {
                throw new UserException(UserEnums.INVALID_INVITATION_CODE);
            }
            //拿到父级关系链
            UserRelation parent = findByUserId(userInfo.getUserId());
            userRelation.setPid(parent.getUserId());
            userRelation.setRelationChain(parent.getRelationChain() + "," + userId);
            userRelation.setTreeDepth(parent.getTreeDepth() + 1);
        }

        String id = UUID.randomUUID().toString();
        userRelation.setId(id);
        userRelation.setUserId(userId);
        userRelation.setCreateTime(new Date());
        return userRelationMapper.insertSelective(userRelation);
    }

    @Override
    public int insertRelationChainByNotInvited(String userId) {
        UserRelation userRelation = new UserRelation();
        String id = UUID.randomUUID().toString();
        userRelation.setId(id);
        userRelation.setUserId(userId);
        userRelation.setRelationChain( userId);
        userRelation.setCreateTime(new Date());
        return userRelationMapper.insert(userRelation);
    }

    @Override
    public UserRelation findByUserId(String userId) {
        UserRelation where = new UserRelation();
        where.setUserId(userId);
        return userRelationMapper.selectOne(where);
    }
}
