package com.blockchain.server.yyyf.controller;

import com.blockchain.common.base.constant.YyyfConstant;
import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.common.base.dto.YyyfUserDto;
import com.blockchain.common.base.util.YyyfUserDtoUtils;
import com.blockchain.server.yyyf.controller.api.ExamApi;
import com.blockchain.server.yyyf.dto.AssessUserDto;
import com.blockchain.server.yyyf.dto.UserTimeMoneyDto;
import com.blockchain.server.yyyf.dto.YyyfLoginDto;
import com.blockchain.server.yyyf.entity.AssessUser;
import com.blockchain.server.yyyf.entity.AssessUserTarget;
import com.blockchain.server.yyyf.entity.YyyfMoney;
import com.blockchain.server.yyyf.enums.YyyfEnums;
import com.blockchain.server.yyyf.exceprion.YyyfException;
import com.blockchain.server.yyyf.service.AssessUserService;
import com.blockchain.server.yyyf.service.AssessUserTargetService;
import com.blockchain.server.yyyf.service.YyyfMoneyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;
import java.util.concurrent.TimeUnit;

/**
 * @author huangxl
 * @data 2019/2/21 15:06
 * 学生成绩控制器
 */
@RestController
@Api(ExamApi.CONTROLLER_API)
@RequestMapping("/exam")
public class ExamController {

    private static Logger LOG = LoggerFactory.getLogger(ExamController.class);

    @Autowired
    private AssessUserTargetService assessUserTargetService;
    @Autowired
    private AssessUserService assessUserService;
    @Autowired
    private YyyfMoneyService yyyfMoneyService;

    @Autowired
    private RedisTemplate redisTemplate;

    @PostMapping("/saveAnswer")
    @ApiOperation(value = ExamApi.SaveAnswer.METHOD_NAME,
            notes = ExamApi.SaveAnswer.METHOD_NOTE)
    public ResultDTO<Integer> saveAnswer(@ApiParam(ExamApi.SaveAnswer.METHOD_YYYF_USER_ID) @RequestParam(name = "yyyfUserId") Integer yyyfUserId,
                                         @ApiParam(ExamApi.SaveAnswer.METHOD_ASSESS_USER_ID) @RequestParam(name = "assessUserId") String assessUserId,
                                         @ApiParam(ExamApi.SaveAnswer.METHOD_STUDENT_TASK_RATE) @RequestParam(name = "taskRate") Double taskRate,
                                         @ApiParam(ExamApi.SaveAnswer.METHOD_STARGE_ID) @RequestParam(name = "code") String code) {

        AssessUserTarget assessUserTarget = this.assessUserTargetService.selectByAssessUserAndCode(assessUserId, code);
        Integer answer = -1;
        if (assessUserTarget != null) {
            assessUserTarget.setStudentRate(taskRate);
            int compareAnswer = taskRate.compareTo(assessUserTarget.getTaskRate());
            //最终指标结果一致
            if (assessUserTarget.getTaskOperate().intValue() == compareAnswer) {
                answer = 0;
            } else {
                answer = 1;
            }
            assessUserTarget.setAnswer(answer);
            this.assessUserTargetService.update(assessUserTarget);

            String caseKey = YyyfConstant.CASE_KEY + yyyfUserId;
            if (redisTemplate.hasKey(caseKey))
                redisTemplate.expire(caseKey, 2, TimeUnit.HOURS);

        } else {
            throw new YyyfException(YyyfEnums.NO_INDICATORS_FOUND);
        }
        return ResultDTO.requstSuccess(answer);
    }

    @SneakyThrows
    @GetMapping("/submit")
    @ApiOperation(value = ExamApi.Submit.METHOD_NAME,
            notes = ExamApi.Submit.METHOD_NOTE)
    public ResultDTO<AssessUserDto> submit(HttpServletRequest request) {

        YyyfUserDto yyyfUserDto = YyyfUserDtoUtils.getYyyfUserDto(request, redisTemplate);
        String assessUserId = yyyfUserDto.getAssessUserId();
        AssessUserDto assessUserDto = this.assessUserService.submit(assessUserId);

        if(assessUserDto.getAchieveMentType()==1){
            redisTemplate.delete(YyyfConstant.CASE_KEY + assessUserDto.getUserId());
        }
        return ResultDTO.requstSuccess(assessUserDto);
    }

    @GetMapping("/reStart")
    @ApiOperation(value = ExamApi.Restart.METHOD_NAME,
            notes = ExamApi.Restart.METHOD_NOTE)
    public ResultDTO<YyyfLoginDto> reStart(HttpServletRequest request) {

        YyyfUserDto yyyfUserDto = YyyfUserDtoUtils.getYyyfUserDto(request, redisTemplate);
        String assessUserId = yyyfUserDto.getAssessUserId();
        AssessUser assessUser = this.assessUserService.selectById(assessUserId);
        if(assessUser.getAchieveMentType()==1){
            throw new YyyfException(YyyfEnums.FODBID_EXAM_RESTART);
        }
        Date now=new Date();
        //系统账户初始化
        String tel=new SimpleDateFormat("yyMMddHHmmss").format(now).concat(String.valueOf(yyyfUserDto.getUserId()));
        //第一次进入进行注册
        String code=String.format("%06d",new Random().nextInt(1000000));
        redisTemplate.opsForValue().set(YyyfConstant.REGISTER_KEY.concat(tel),code,YyyfConstant.TIME_OUT, TimeUnit.MINUTES);
        //本系统密码
        String password=String.format("%08d",new Random().nextInt(10000000));

        assessUser.setTel(tel);
        assessUser.setPassword(password);
        assessUser.setStartTime(now);
        assessUser.setEndTime(null);


        this.assessUserService.update(assessUser);

        //区块链原始金额还原
        YyyfMoney yyyfMoney=new YyyfMoney();
        yyyfMoney.setAssessUserId(assessUser.getId());
        yyyfMoney.setAliPay(assessUser.getAlipayInitMoney());
        yyyfMoney.setWxPay(assessUser.getWechatInitMoney());
        yyyfMoney.setBankCard(assessUser.getBankcardInitMoney());
        yyyfMoney.setBtc(BigDecimal.ZERO);
        yyyfMoney.setEth(BigDecimal.ZERO);
        yyyfMoney.setUsdt(BigDecimal.ZERO);
        yyyfMoney.setEos(BigDecimal.ZERO);
        yyyfMoney.setUpdateTime(now);

        this.yyyfMoneyService.update(yyyfMoney);

        this.assessUserTargetService.restoreByAssessUserId(assessUserId);

        YyyfLoginDto yyyfLoginDto = new YyyfLoginDto();
        yyyfLoginDto.setCode(code);
        yyyfLoginDto.setTel(tel);
        yyyfLoginDto.setPassword(password);

        return ResultDTO.requstSuccess(yyyfLoginDto);
    }



    @SneakyThrows
    @GetMapping("/getUserTimeMoneyDto")
    @ApiOperation(value = ExamApi.GetUserTimeMoneyDto.METHOD_NAME,
            notes = ExamApi.GetUserTimeMoneyDto.METHOD_NOTE)
    public ResultDTO<UserTimeMoneyDto> getUserTimeMoneyDto(HttpServletRequest request) {

        YyyfUserDto yyyfUserDto = YyyfUserDtoUtils.getYyyfUserDto(request, redisTemplate);
        String assessUserId = yyyfUserDto.getAssessUserId();
        UserTimeMoneyDto userTimeMoneyDto =this.assessUserService.getUserTimeMoneyDtoByAssessUserId(assessUserId);
        return ResultDTO.requstSuccess(userTimeMoneyDto);
    }
}
