package com.blockchain.server.yyyf.controller;

import com.blockchain.common.base.constant.YyyfConstant;
import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.common.base.dto.TokenDTO;
import com.blockchain.common.base.dto.YyyfUserDto;
import com.blockchain.common.base.util.RSACoderUtils;
import com.blockchain.server.train.dto.ExamPaperDto;
import com.blockchain.server.train.dto.TrainCaseTargetDto;
import com.blockchain.server.yyyf.controller.api.LoginApi;
import com.blockchain.server.yyyf.dto.*;
import com.blockchain.server.yyyf.entity.AssessUser;
import com.blockchain.server.yyyf.service.AssessUserService;
import com.blockchain.server.yyyf.service.AssessUserTargetService;
import com.blockchain.server.yyyf.utils.IdGenerator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import sun.misc.BASE64Decoder;

import javax.validation.Valid;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author huangxl
 * @data 2019/2/21 15:06
 * 用户注册, 登录控制器
 */
@RestController
@Api(LoginApi.CONTROLLER_API)
@RequestMapping("/loginFromYyyf")
public class LoginController {

    private static final Logger LOG = LoggerFactory.getLogger(LoginController.class);


    @Autowired
    private AssessUserService assessUserService;

    @Autowired
    private RedisTemplate redisTemplate;

    @PostMapping("/login")
    @ApiOperation(value = LoginApi.YyyfLogin.METHOD_NAME,
            notes = LoginApi.YyyfLogin.METHOD_NOTE)
    public ResultDTO<YyyfLoginDto> loginyyyf(@ApiParam(LoginApi.YyyfLogin.METHOD_API_YYYFLOGINREQUESTVO) @Valid YyyfLoginRequestVo yyyfLoginRequestVo) {


        //存到redis中用于以渔有方用户与智信链用户进行绑定

        //登录返回的Data
        YyyfLoginDto yyyfLoginDto = new YyyfLoginDto();
        yyyfLoginDto.setStatus(0);
        String userName = null;
        try {
            userName = new String(new BASE64Decoder().decodeBuffer(yyyfLoginRequestVo.getUserName().replaceAll(" ", "+")), "UTF-8");
        } catch (IOException e) {
            LOG.error(e.getMessage(), e);
        }
        //根据案例id，考核id封装相关信息。
        //1、从缓存中取案列相关
        //0 考试 1 练习 2 不计分 3 教师、管理员的案例管理
        int reqType = yyyfLoginRequestVo.getReqType().intValue();
        Integer classId = yyyfLoginRequestVo.getClassId();
        Integer userId = yyyfLoginRequestVo.getUserId();


        //以渔有方考核id
        String examId = yyyfLoginRequestVo.getReqId();
        //试卷id（项目id）
        String examPaperId = yyyfLoginRequestVo.getCaseId();

        YyyfUserDto yyyfUserDto = new YyyfUserDto();
        yyyfUserDto.setUserName(userName);
        yyyfUserDto.setUserId(userId);
        yyyfUserDto.setUserType(yyyfLoginRequestVo.getUserType());

        //做考试和练习
        if (0 == reqType || 1 == reqType) {
            if (0 == reqType) {
                //如果考试他提交了，库里有信息了，不允许他再次进入了。
                if (this.assessUserService.selectScore(examId, userId) != null) {
                    yyyfLoginDto.setStatus(2);
                    yyyfLoginDto.setMsg(YyyfConstant.SUBMITTED);
                    return handleAfterLogin(yyyfLoginDto, yyyfUserDto);
                }
            }
            AssessUser assessUser = this.assessUserService.selectAssessUserByUserIdAndissueIdForNotEnd(userId, examId);
            if (assessUser == null) {
                Date now = new Date();
                String tel=new SimpleDateFormat("yyMMddHHmmss").format(now).concat(String.valueOf(userId));

                //第一次进入进行注册
                String code=String.format("%06d",new Random().nextInt(1000000));
                redisTemplate.opsForValue().set(YyyfConstant.REGISTER_KEY.concat(tel),code,YyyfConstant.TIME_OUT, TimeUnit.MINUTES);
                yyyfLoginDto.setCode(code);

                String password=String.format("%08d",new Random().nextInt(10000000));
                String assessUserId = IdGenerator.uuid();
                assessUser = new AssessUser();
                assessUser.setId(assessUserId);
                assessUser.setAchieveMentType(reqType == 0 ? 1 : 0);
                assessUser.setCaseId(yyyfLoginRequestVo.getCaseId());
                assessUser.setClassId(String.valueOf(classId));
                assessUser.setCourseId(yyyfLoginRequestVo.getCourserId());
                assessUser.setIssueId(examId);
                assessUser.setSsPractiseId(yyyfLoginRequestVo.getCaseId());
                assessUser.setStartTime(now);
                assessUser.setUserId(userId);
                assessUser.setUserName(userName);
                assessUser.setUserType(yyyfLoginRequestVo.getUserType());
                assessUser.setTel(tel);
                assessUser.setPassword(password);

                //获取案例信息
                HashOperations<String, String, ExamPaperDto> examPaperOpsForHash = redisTemplate.opsForHash();
                ExamPaperDto examPaperDto = examPaperOpsForHash.get(YyyfConstant.EXAM_PAPER_KEY, examPaperId);
                List<TrainCaseTargetDto> trainCaseTargeList = examPaperDto.getTrainCaseTargetList();
                assessUser.setWechatInitMoney(examPaperDto.getWechatInitMoney());
                assessUser.setAlipayInitMoney(examPaperDto.getAlipayInitMoney());
                assessUser.setBankcardInitMoney(examPaperDto.getBankcardInitMoney());
                assessUser.setDigitwalletInitMoney(examPaperDto.getDigitwalletInitMoney());
                this.assessUserService.initAssessUser(assessUser, trainCaseTargeList);
            }



            yyyfUserDto.setAssessUserId(assessUser.getId());
            yyyfLoginDto.setTel(assessUser.getTel());
            yyyfLoginDto.setPassword(assessUser.getPassword());
            yyyfLoginDto.setNickName(userName);

            HashOperations<String, String, ExamDto> examDtoOpsForHash = redisTemplate.opsForHash();

            //将参加考试的人员缓存
            HashOperations<String, String, List<Integer>> studentOpsForHash = redisTemplate.opsForHash();
            //参加考试的人员id
            List<Integer> joinStuIds = null;

            //考试id和案列id缓存
            String caseKey = YyyfConstant.CASE_KEY + userId;

            //判断考试缓存中是否存在这张卷子
            if (0 == reqType) {
                ExamDto examDto = examDtoOpsForHash.get(YyyfConstant.EXAM_KEY, examId);
                if (examDto == null) {
                    examDto = new ExamDto();
                    examDto.setExamId(yyyfLoginRequestVo.getReqId());
                    examDto.setBeginTime(yyyfLoginRequestVo.getBeginTime());
                    examDto.setEndTime(yyyfLoginRequestVo.getEndTime());
                    examDto.setType(reqType);
                    examDto.setExamPaperId(examPaperId);
                    examDtoOpsForHash.put(YyyfConstant.EXAM_KEY, examId, examDto);
                }
                yyyfLoginDto.setRemainingTime((examDto.getEndTime().getTime() - new Date().getTime() )/ 1000);
                joinStuIds = studentOpsForHash.get(YyyfConstant.JOIN_EXAM_STU_IDS_KEY, examId);
                if (joinStuIds == null) {
                    joinStuIds = new ArrayList<Integer>();
                }

                Map<String, String> map = new HashMap<String, String>();
                map.put("examPaperId", examDto.getExamPaperId());
                map.put("examId", examDto.getExamId());

                redisTemplate.opsForValue().set(caseKey, map);
                redisTemplate.expire(caseKey, 30, TimeUnit.DAYS);

                //如果等于0
                if (joinStuIds.stream()
                        .filter(stuIds -> (stuIds.equals(userId))).count() == 0) {
                    joinStuIds.add(userId);
                    studentOpsForHash.put(YyyfConstant.JOIN_EXAM_STU_IDS_KEY, examId, joinStuIds);
                } else {
                    yyyfLoginDto.setStatus(3);
                    yyyfLoginDto.setMsg(YyyfConstant.NO_ENTRY_INTO_PRACTICE);
                    return handleAfterLogin(yyyfLoginDto, yyyfUserDto);
                }
            } else {
                String jioinExamId=null;
                if (redisTemplate.hasKey(caseKey)) {
                    Map<String, String> examMsgMap = (Map<String, String>) redisTemplate.opsForValue().get(caseKey);
                    jioinExamId= examMsgMap.get("examId");
                    joinStuIds = studentOpsForHash.get(YyyfConstant.JOIN_EXAM_STU_IDS_KEY,jioinExamId);
                }
                //缓存是考试，要进入练习，则不让进入
                if (joinStuIds != null && joinStuIds.stream()
                        .filter(stuIds -> (stuIds.equals(userId))).count() != 0) {
                    yyyfLoginDto.setStatus(3);
                    yyyfLoginDto.setMsg(YyyfConstant.NO_ENTRY_INTO_PRACTICE);
                    ExamDto examDto = examDtoOpsForHash.get(YyyfConstant.EXAM_KEY, examId);
                    yyyfLoginDto.setRemainingTime((examDto.getEndTime().getTime() - new Date().getTime() )/ 1000);

                    Map<String, String> map = new HashMap<String, String>();
                    map.put("examPaperId", examDto.getExamPaperId());
                    map.put("examId", examDto.getExamId());

                    redisTemplate.opsForValue().set(caseKey, map);
                    redisTemplate.expire(caseKey, 30, TimeUnit.DAYS);

                    return handleAfterLogin(yyyfLoginDto, yyyfUserDto);
                }


            }
            //删除强制提醒的缓存
            // redisTemplate.delete(YyyfConstant.ANSER_KEY + userId);
            yyyfLoginDto.setStatus(1);
        }



        return handleAfterLogin(yyyfLoginDto, yyyfUserDto);
    }


    /**
     * 登录成功之后的处理
     */
    private ResultDTO handleAfterLogin(YyyfLoginDto yyyfLoginDto, YyyfUserDto yyyfUserDto) {
        int status = yyyfLoginDto.getStatus();
        String token = generateToken(String.valueOf(yyyfUserDto.getUserId()), System.currentTimeMillis(),
                YyyfConstant.YYYF_ZHIXINLIAN_USER_KEY);
        yyyfLoginDto.setYyyfUserToken(token);
        if (status != 2) {
            //到时看要不要设定时长
            redisTemplate.opsForValue().set(token, yyyfUserDto);
        }
        return ResultDTO.requstSuccess(yyyfLoginDto);
    }

    /**
     * 生成token返回前端
     *
     * @param tel       手机号
     * @param timestamp 时间撮
     * @return token
     */
    private String generateToken(String tel, long timestamp, String tokenType) {
        TokenDTO tokenDTO = new TokenDTO();
        tokenDTO.setTel(tel);
        tokenDTO.setTimestamp(timestamp);
        tokenDTO.setTokenType(tokenType);
        return RSACoderUtils.encryptToken(tokenDTO);
    }

}
