package com.blockchain.server.yyyf.controller;

import com.alibaba.fastjson.JSONObject;
import com.blockchain.common.base.constant.YyyfConstant;
import com.blockchain.common.base.dto.ResultDTO;
import com.blockchain.server.train.dto.ExamPaperDto;
import com.blockchain.server.train.dto.TrainCaseTargetDto;
import com.blockchain.server.yyyf.controller.api.TeachingGradeApi;
import com.blockchain.server.yyyf.dto.ExamDetailsDto;
import com.blockchain.server.yyyf.dto.OverviewDto;
import com.blockchain.server.yyyf.dto.ReExamOverviewDto;
import com.blockchain.server.yyyf.dto.StudentTargetErrorStatisticsDto;
import com.blockchain.server.yyyf.entity.AssessUser;
import com.blockchain.server.yyyf.service.AssessUserService;
import com.blockchain.server.yyyf.service.AssessUserTargetService;
import com.blockchain.server.yyyf.utils.ResponseUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;
import sun.misc.BASE64Decoder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author huan.xu
 * @version 1.0
 * @className TeachingGradeController
 * @description 练习、考核成绩Controller
 * @date 2019-11-05 16:23
 */
@RestController
@Api(TeachingGradeApi.CONTROLLER_API)
@RequestMapping("/teachingGrade")
public class TeachingGradeController {
    protected Logger logger = LoggerFactory.getLogger(TeachingGradeController.class);

    @Autowired
    private RedisTemplate redisTemplate;

    private static final String SYS_CODE = "102";

    @Autowired
    private AssessUserService assessUserService;

    @Autowired
    private AssessUserTargetService assessUserTargetService;

    // @Autowired
    // private ScoreSynchroService scoreSynchroService;


    /**
     * @description 教师获取考核概况
     * @author huan.xu
     * @date  2020-05-19 10:47:29
     * @param [issueId, totalNum, caseId, response]
     * @return void
     **/
    @GetMapping(value = "/getGradeData", produces = {"application/text;charset=UTF-8"})
    @ResponseBody
    @SuppressWarnings("all")
    @ApiOperation(value = TeachingGradeApi.GetGradeData.METHOD_NAME,
            notes = TeachingGradeApi.GetGradeData.METHOD_NOTE)
    public void getGradeData(@ApiParam(TeachingGradeApi.GetGradeData.METHOD_ISSUE_ID) @RequestParam(name = "issueId") String issueId,
                             @ApiParam(TeachingGradeApi.GetGradeData.METHOD_TOTAL_NUM) @RequestParam(name = "totalNum") Integer totalNum,
                             @ApiParam(TeachingGradeApi.GetGradeData.METHOD_CASE_ID) @RequestParam(name = "caseId") String caseId,
                             HttpServletResponse response) {

        JSONObject json = new JSONObject();
        try {

            int answerCount = assessUserService.selectCountByByIssueId(issueId);
            //获取案例信息
            HashOperations<String, String, ExamPaperDto> examPaperOpsForHash = redisTemplate.opsForHash();
            ExamPaperDto examPaperDto = examPaperOpsForHash.get(YyyfConstant.EXAM_PAPER_KEY, caseId);
            List<TrainCaseTargetDto> trainCaseTargeList = examPaperDto.getTrainCaseTargetList();

            Map<String, Object> wrongStatistics = null;//错误信息
            //无人考试 时封装错误率
            if (answerCount == 0) {

                wrongStatistics = new HashMap<>();

                Map<String, Object> maxErrorRateInfo = new HashMap<>();//最大错误率信息记录
                Map<String, Object> minErrorRateInfo = new HashMap<>();//最小错误率信息记录
                List<String> taskNames = new ArrayList<>();
                List<Double> errorRates = new ArrayList<>();
                for (TrainCaseTargetDto trainCaseTargetDto : trainCaseTargeList) {
                    taskNames.add(trainCaseTargetDto.getName());
                    errorRates.add(Double.valueOf(100));
                    if (maxErrorRateInfo.size() == 0) {
                        maxErrorRateInfo.put("headCount", totalNum);
                        maxErrorRateInfo.put("taskName", trainCaseTargetDto.getName());
                        maxErrorRateInfo.put("errorRate", Double.valueOf(100));
                        maxErrorRateInfo.put("projectName", trainCaseTargetDto.getName());
                        maxErrorRateInfo.put("errorCount", totalNum);
                        continue;
                    }
                    if (minErrorRateInfo.size() == 0) {
                        minErrorRateInfo.put("headCount", totalNum);
                        minErrorRateInfo.put("taskName", trainCaseTargetDto.getName());
                        minErrorRateInfo.put("errorRate", Double.valueOf(100));
                        minErrorRateInfo.put("projectName", trainCaseTargetDto.getName());
                        minErrorRateInfo.put("errorCount", totalNum);
                        continue;
                    }
                }
                wrongStatistics.put("maxErrorRateInfo", maxErrorRateInfo);
                wrongStatistics.put("minErrorRateInfo", minErrorRateInfo);
                wrongStatistics.put("taskNames", taskNames);
                wrongStatistics.put("errorRates", errorRates);

            } else {
                wrongStatistics = getWrongStatistics(issueId, totalNum, trainCaseTargeList);
            }


            json.put("result", true);
            json.put("wrongStatistics", wrongStatistics);
        } catch (Exception e) {
            json.put("result", false);
            json.put("msg", "获取成绩报表数据失败");
            logger.error("获取成绩报表数据失败: " + e.getMessage(), e);
        }

        ResponseUtils.renderJson(response, json.toString());
    }

    /**
     * @param [issueId, notExamNum]
     * @return java.util.Map<java.lang.String, java.lang.Object>
     * @description 封装错误率
     * @author huan.xu
     * @date 2019-11-08 18:27:22
     **/
    private Map<String, Object> getWrongStatistics(String issueId, Integer totalNum, List<TrainCaseTargetDto> trainCaseTargeList) {
        Map<String, Object> wrongStatistics = new HashMap<String, Object>();
        List<StudentTargetErrorStatisticsDto> studentTargetErrorStatisticsDtoLis = this.assessUserTargetService.getStudentTargetErrorStatisticsDtoListByIssueId(issueId);
        //最大错误率信息记录
        Map<String, Object> maxErrorRateInfo = null;
        //最小错误率信息记录
        Map<String, Object> minErrorRateInfo = null;
        //记录考核点名称
        String[] taskNames = new String[trainCaseTargeList.size()];
        //记录错误率
        double[] errorRates = new double[trainCaseTargeList.size()];


        int i = 0;
        for (TrainCaseTargetDto trainCaseTargetDto : trainCaseTargeList) {
            String targetName = trainCaseTargetDto.getName();
            taskNames[i] = targetName;
            //错误人数
            int errorCount = 0;
            for (StudentTargetErrorStatisticsDto studentTargetErrorStatisticsDto : studentTargetErrorStatisticsDtoLis) {
                //答题错误记录人数
                if (studentTargetErrorStatisticsDto.getTargetId().equals(trainCaseTargetDto.getId())) {
                    errorCount++;
                    break;
                }
            }
            //计算错误率
            double errorRate = (double) (errorCount) / (double) (totalNum) * 100;
            //四舍五入保留两位小数
            errorRate = Double.valueOf(String.format("%.2f", errorRate));
            errorRates[i] = errorRate;
            //处理最大和最小错误率信息
            if (maxErrorRateInfo == null) {
                maxErrorRateInfo = new HashMap<String, Object>();
                maxErrorRateInfo.put("projectName", targetName);
                maxErrorRateInfo.put("taskName", targetName);
                maxErrorRateInfo.put("headCount", totalNum);
                maxErrorRateInfo.put("errorCount", errorCount);
                maxErrorRateInfo.put("errorRate", errorRate);
            } else {
                //上一次最大错误率
                double prevMaxErrorRate = Double.valueOf(maxErrorRateInfo.get("errorRate").toString());
                if (errorRate > prevMaxErrorRate) {
                    //本次错误率大于上次，则记录本次信息
                    maxErrorRateInfo.put("projectName", targetName);
                    maxErrorRateInfo.put("taskName", targetName);
                    maxErrorRateInfo.put("headCount", totalNum);
                    maxErrorRateInfo.put("errorCount", errorCount);
                    maxErrorRateInfo.put("errorRate", errorRate);
                }
            }
            if (minErrorRateInfo == null) {
                minErrorRateInfo = new HashMap<String, Object>();
                minErrorRateInfo.put("projectName", targetName);
                minErrorRateInfo.put("taskName", targetName);
                minErrorRateInfo.put("headCount", totalNum);
                minErrorRateInfo.put("errorCount", errorCount);
                minErrorRateInfo.put("errorRate", errorRate);
            } else {
                //上一次最小错误率
                double prevMinErrorRate = Double.valueOf(minErrorRateInfo.get("errorRate").toString());
                if (prevMinErrorRate > errorRate) {
                    //本次错误率小于上次最小错误率，则记录本次信息
                    minErrorRateInfo.put("projectName", targetName);
                    minErrorRateInfo.put("taskName", targetName);
                    minErrorRateInfo.put("headCount", totalNum);
                    minErrorRateInfo.put("errorCount", errorCount);
                    minErrorRateInfo.put("errorRate", errorRate);
                }
            }
            i++;
        }
        wrongStatistics.put("maxErrorRateInfo", maxErrorRateInfo);
        wrongStatistics.put("minErrorRateInfo", minErrorRateInfo);
        wrongStatistics.put("taskNames", taskNames);
        wrongStatistics.put("errorRates", errorRates);
        return wrongStatistics;
    }

    @PostMapping("/getExamOverview")
    @SuppressWarnings("all")
    @ApiOperation(value = TeachingGradeApi.GetExamOverview.METHOD_NAME,
            notes = TeachingGradeApi.GetExamOverview.METHOD_NOTE)
    public ResultDTO<OverviewDto> getExamOverview(@ApiParam(TeachingGradeApi.GetExamOverview.METHOD_REEXAMOVERVIEWDTO) ReExamOverviewDto reExamOverviewDto) {
        OverviewDto overviewDto = new OverviewDto();
        try {
            AssessUser assessUser = null;
            if ("1".equals(reExamOverviewDto.getReqType())) { //考试
                assessUser = this.assessUserService.getAssessUserByIssueIdAndUserIdForExam(reExamOverviewDto.getIssueId(), reExamOverviewDto.getUserId());
            } else {
                List<AssessUser> assessUserList = this.assessUserService.getAssessUserListByIssueIdAndUserId(reExamOverviewDto.getIssueId(), reExamOverviewDto.getUserId());
                List<String> assessUserIdsList = new ArrayList<>();
                for (AssessUser assessUserSingle : assessUserList) {
                    if (assessUser == null) {
                        assessUser = assessUserSingle;
                    }
                    assessUserIdsList.add(assessUserSingle.getId());
                }
                overviewDto.setAssessUserIdsList(assessUserIdsList);
            }
            double duration = assessUser.getEndTime().getTime() - assessUser.getStartTime().getTime();
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append(String.format("%02d", (int) Math.floor(duration / 1000 / 60 / 60)))
                    .append(":").append(String.format("%02d", (int) Math.floor(duration / 1000 / 60 % 60)))
                    .append(":").append(String.format("%02d", (int) Math.floor(duration / 1000 % 60)));
            //考试花费时间
            overviewDto.setUseTime(stringBuilder.toString());
            //考试开始时间
            overviewDto.setExamDate(new SimpleDateFormat("yyyy/MM/dd").format(assessUser.getStartTime()));

            overviewDto.setAssessUserId(assessUser.getId());
            //学生姓名
            overviewDto.setUserName(new String(new BASE64Decoder().decodeBuffer(reExamOverviewDto.getUserName().replaceAll(" ", "+")), "UTF-8"));
            //班级名称
            overviewDto.setClassName(new String(new BASE64Decoder().decodeBuffer(reExamOverviewDto.getClassName().replaceAll(" ", "+")), "UTF-8"));

            //考试成绩
            overviewDto.setTotalScore(assessUser.getTotalScore());

        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return ResultDTO.requstSuccess(overviewDto);
    }

    @GetMapping("/getExamDetails")
    @SuppressWarnings("all")
    @ApiOperation(value = TeachingGradeApi.GetExamDetails.METHOD_NAME,
            notes = TeachingGradeApi.GetExamDetails.METHOD_NOTE)
    public ResultDTO<ExamDetailsDto> getExamDetails(@ApiParam(TeachingGradeApi.GetExamDetails.METHOD_ASSESSUSER_ID)@RequestParam("assessUserId") String assessUserId,
                                                    @ApiParam(TeachingGradeApi.GetExamDetails.METHOD_IS_FIRST)@RequestParam("isFirst") int isFirst) {
        ExamDetailsDto examDetailsDto = new ExamDetailsDto();
        examDetailsDto.setIsFirst(isFirst);
        try {
            if (isFirst != 0) {
                AssessUser assessUser = this.assessUserService.selectById(assessUserId);
                double duration = assessUser.getEndTime().getTime() - assessUser.getStartTime().getTime();
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append(String.format("%02d", (int) Math.floor(duration / 1000 / 60 / 60)))
                        .append(":").append(String.format("%02d", (int) Math.floor(duration / 1000 / 60 % 60)))
                        .append(":").append(String.format("%02d", (int) Math.floor(duration / 1000 % 60)));
                //考试花费时间
                examDetailsDto.setUseTime(stringBuilder.toString());
                //考试开始时间
                examDetailsDto.setExamDate(new SimpleDateFormat("yyyy/MM/dd").format(assessUser.getStartTime()));
                //考试成绩
                examDetailsDto.setTotalScore(assessUser.getTotalScore());
            }
            examDetailsDto.setAssessUserTargetList(this.assessUserTargetService.selectAssessUserTargetByAssessUserId(assessUserId));
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return ResultDTO.requstSuccess(examDetailsDto);
    }


    /**
     * @param [issueId, authorization]
     * @return java.util.Map<java.lang.String, java.lang.Object>
     * @description 重新成绩同步
     * @author huan.xu
     * @date 2019-11-08 18:39:55
     **/
    @GetMapping("/studyGrade/reSynchroze")
    @ResponseBody
    @SuppressWarnings("all")
    public Map<String, Object> reSynchroze(@ApiParam("考试id") @RequestParam("issueId") String issueId,
                                           @ApiParam("验证码") @RequestParam("authorization") String authorization) {
        Map<String, Object> result = new HashMap<>();
        result.put("status", 0);
        result.put("msg", "系统错误");
        try {
            if ("87DIVy348Oxzj3ha".equals(authorization)) {

                this.assessUserService.synchronousScore(issueId);
                result.put("status", 1);
                result.put("msg", "同步成功");
            } else {
                result.put("msg", "验证错误");
            }
        } catch (Exception e) {
            result.put("msg", "同步失败,请进行后台检查");
            logger.error(e.getMessage(), e);
        }
        return result;
    }


    @GetMapping("/export")
    @ResponseBody
    @ApiIgnore
    public void yhExport(HttpServletRequest request, HttpServletResponse response) {/*
        String examName = request.getParameter("examName");
        String className = request.getParameter("className");
        String stuNo = request.getParameter("stuNo");
        String totalScore = request.getParameter("totalScore");
        String useTime = request.getParameter("useTime");
        String examDate = request.getParameter("examDate");
        String assessUserId = request.getParameter("assessUserId");
        String userName = request.getParameter("userName");
        String caseInfo = request.getParameter("caseInfo");
        String caseId = request.getParameter("caseId");
     //   TifPro tifPro = JedisUtil.getMapValue(JedisUtil.bulidKey(Const.TIFPRO_KEY), caseId, TifPro.class);

        //设置导出文件名称
        String fileName = ((new SimpleDateFormat("yyyyMMdd")).format(new Date()))+"-"+userName+ "-考核详情.xls";
        response.setContentType("application/x-download");//不同类型的文件对应不同的MIME类型
        try {
            if (request.getHeader("USER-AGENT").toLowerCase().indexOf("firefox") > 0) {
                fileName = new String(fileName.getBytes("GB2312"), "ISO-8859-1");
            } else {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
        HSSFWorkbook wb = new HSSFWorkbook();
        HSSFSheet sheet = wb.createSheet("成绩详情");  //创建table工作薄
        Object[][] datas = {
                {"学生姓名：" + userName,
                        "班级：" + className,
                        "学号：" + stuNo,
                        "成绩：" + totalScore+" 分",
                        "日期：" + examDate, "用时：" + useTime},
                {"考核名称：" + examName,"","","","",""},
                {"项目名称：" + tifPro.getProjectName(),"","","","",""},
                {"项目背景：" + caseInfo,"","","","",""},
                {"成绩解析：","","","","",""}};

        List<Object[]> cellDate = new ArrayList<Object[]>();
        for (Object[] data : datas) {
            cellDate.add(data);
        }

        List<PractiseProDto> practiseProDtos = this.assessUserService.selectPractiseProList(assessUserId);

        //合并单元格参数
        List<int[]> regionList = new ArrayList<int[]>();
        regionList.add(new int[]{1, 1, 0, 5});
        regionList.add(new int[]{2, 2, 0, 5});
        regionList.add(new int[]{3, 3, 0, 5});
        regionList.add(new int[]{4, 4, 0, 5});


        int reginonIndex = 4;

        HSSFRow row;
        HSSFCell cell;


        //regionList.add(new int[]{5,5,1,2});


        //靠左的行索引
        HashSet leftRowIndexSet=new HashSet();
        leftRowIndexSet.add(reginonIndex);
        for (PractiseProDto practiseProDto : practiseProDtos) {
            reginonIndex++;
            leftRowIndexSet.add(reginonIndex);

            regionList.add(new int[]{reginonIndex, reginonIndex, 0, 5});
            Object[] trx = {practiseProDto.getProName()+("(得分：")+(practiseProDto.getProjectScore())+(")").toString()};
            cellDate.add(trx);//交易层次

            //
            Object[] trxTitle = {"序号", "考核点", "正确答案", "学生答案", "考核点分值", "得分"};
            cellDate.add(trxTitle);
            reginonIndex++;


        	*//*Object[] assess = {"",taskVo.getTaskName(),"",(taskVo.getAnswer()==0?"对":"错"),""};
    			cellDate.add(assess);*//*
            List<PractiseTaskDto> taskList = practiseProDto.getTaskList();

            if (taskList != null && taskList.size() > 0) {
                int i = 1;
                for (PractiseTaskDto taskDto : taskList) {
                    String rightAnswer = taskDto.getRightAnswer();
                    if (rightAnswer == null) {
                        rightAnswer = "-";
                    }
                    String stuAnswer = taskDto.getStuAnswer();
                    if (stuAnswer == null) {
                        stuAnswer = "-";
                    }

                    BigDecimal studentScore = BigDecimal.ZERO;

                    if (taskDto.getAnswer().intValue() == 0) {
                        studentScore = taskDto.getTaskScore();
                    }
                    Object[] assess = {i, taskDto.getTaskName(), rightAnswer, stuAnswer, taskDto.getTaskScore(), studentScore};
                    cellDate.add(assess);
                    reginonIndex++;
                    i++;
                }
            } else {
                Object[] assess = {"-", "-", "-", "-", "-", "-"};
                cellDate.add(assess);
                reginonIndex++;
            }

			*//*regionList.add(new int[]{maxRow,maxRow,1,2});
			maxRow ++;*//*
        }

        int size = cellDate.size();
        for (int i = 0; i < size; i++) {
            //居中标识
            short alignment=2;
            if(leftRowIndexSet.contains(i)){
                //居左
                alignment=1;
            }
            row = sheet.createRow(i);//创建表格行
            int length = cellDate.get(i).length;
            for (int j = 0; j <length ; j++) {
                cell = row.createCell(j);//根据表格行创建单元格

                String str = String.valueOf(cellDate.get(i)[j]);
                if(i==0&&j==3){
                    setFontColor(wb,cell,str,3,str.length());
                }else if(alignment==1 && i > 4 ){
                    setFontColor(wb,cell,str,str.indexOf("：")+1,str.length()-1);
                }else{
                    cell.setCellValue(str);
                }



                HSSFCellStyle cellStyle = wb.createCellStyle();
                cellStyle.setBorderBottom(HSSFCellStyle.BORDER_THIN);
                cellStyle.setBorderLeft(HSSFCellStyle.BORDER_THIN);
                cellStyle.setBorderTop(HSSFCellStyle.BORDER_THIN);
                cellStyle.setBorderRight(HSSFCellStyle.BORDER_THIN);
                cellStyle.setAlignment(alignment);
                cellStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
                if(i==3)
                    cellStyle.setWrapText(true);
                cell.setCellStyle(cellStyle);
            }
            row = sheet.getRow(i);
            if (i ==3)
                row.setHeightInPoints(320);
            else
                row.setHeightInPoints(30);
        }


        int cellDate1Len = cellDate.get(0).length;

        for (int j = 0; j < cellDate1Len; j++) {
            sheet.setColumnWidth(j, MSExcelUtil.pixel2WidthUnits(180));
        }

        for (int[] regions : regionList) {
            //合并单元格
            CellRangeAddress region = new CellRangeAddress(regions[0], // first row
                    regions[1], // last row
                    regions[2], // first column
                    regions[3] // last column
            );
            sheet.addMergedRegion(region);
        }

        ServletOutputStream sos = null;

        try {
            sos = response.getOutputStream();
            wb.write(sos);
            sos.close();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        } finally {
            if (sos != null) {
                try {
                    sos.close();
                } catch (IOException e) {
                    logger.error(e.getMessage(), e);
                }

            }
        }*/
    }

    /**
     * @param [cell, wb, str, startIndex, endIndex]
     * @return void
     * @description poi实现单元格中部分字体颜色设定
     * @author huan.xu
     * @date 2019-12-10 16:19:48
     **/
    private void setFontColor(HSSFWorkbook wb, HSSFCell cell, String str, int startIndex, int endIndex) {
        HSSFFont font = wb.createFont();
        font.setColor(HSSFColor.RED.index);
        HSSFRichTextString richString = new HSSFRichTextString(str);
        richString.applyFont(startIndex, endIndex, font);
        cell.setCellValue(richString);
    }
}