package com.blockchain.server.yyyf.indicators.handle;

import com.blockchain.server.yyyf.indicators.IndicatorsHandler;
import com.blockchain.server.yyyf.indicators.IndicatorsUtils;
import com.blockchain.server.yyyf.indicators.dto.AssetsReporter;
import com.blockchain.server.yyyf.indicators.dto.LastAndFirstAssetsDto;
import com.blockchain.server.yyyf.service.AssetsReporterService;
import lombok.AllArgsConstructor;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author fengyu
 */
@AllArgsConstructor
@Component(AnnualGrowthRateIndicatorsHandler.HANDLER_NAME + IndicatorsHandler.HANDLER_NAME)
public class AnnualGrowthRateIndicatorsHandler implements IndicatorsHandler {
    public static final String HANDLER_NAME = "AnnualGrowthRate";
    public static final String CACHE_ANNUAL_GROWTH_RATE = "yyyf_indicator_annualGrowthRate";
    public static final String CACHE_KEY_ANNUAL_GROWTH_RATE = "yyyf:annualGrowthRate:";

    private final AssetsReporterService assetsReporterService;

    @Override
    @Cacheable(cacheNames = CACHE_ANNUAL_GROWTH_RATE,
            key = "#root.target.CACHE_KEY_ANNUAL_GROWTH_RATE.concat(#assessUserId)", unless = "#result == null")
    public Float handleIndicator(String assessUserId) {
        List<AssetsReporter> reporterList = assetsReporterService.selectAssetsReporterListByAssessUserId(assessUserId);

        //获取最初、最后交易的信息
        LastAndFirstAssetsDto lastAndFirstAssetsDto = IndicatorsUtils.getLastAndFirstAssetsDto(reporterList);

        AssetsReporter lastAssets = lastAndFirstAssetsDto.getLastAssets();
        AssetsReporter firstAssets = lastAndFirstAssetsDto.getFirstAssets();
        BigDecimal result = lastAssets.getAssets()
                .divide(firstAssets.getAssets(), 5, BigDecimal.ROUND_HALF_UP)
                .pow((int) (360.0 / lastAssets.getDays()))
                .subtract(BigDecimal.valueOf(1));
        return result.multiply(BigDecimal.valueOf(100)).floatValue();
    }
}
