package com.blockchain.server.yyyf.indicators.handle;

import com.blockchain.server.yyyf.indicators.IndicatorsHandler;
import com.blockchain.server.yyyf.indicators.dto.AssetsReporter;
import com.blockchain.server.yyyf.service.AssetsReporterService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import javax.persistence.EntityExistsException;
import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;

/**
 * @author fengyu
 */
@AllArgsConstructor
@Component(MaxDrawdownRateIndicatorsHandler.HANDLER_NAME + IndicatorsHandler.HANDLER_NAME)
public class MaxDrawdownRateIndicatorsHandler implements IndicatorsHandler {
    public static final String HANDLER_NAME = "MaxDrawdownRate";


    private final AssetsReporterService assetsReporterService;

    @Override
    public Float handleIndicator(String assessUserId) {
        List<AssetsReporter> reporterList = assetsReporterService.selectAssetsReporterListByAssessUserId(assessUserId);
        if (reporterList != null && reporterList.size() > 0) {
            //排序
            reporterList.sort(Comparator.comparingLong(AssetsReporter::getDays));

            BigDecimal maxDrawdownRate=BigDecimal.ZERO;
            //上一天的资产总额
            BigDecimal upAssets =BigDecimal.ZERO;
            for (AssetsReporter assetsReporter : reporterList) {
                BigDecimal thisAssets=assetsReporter.getAssets();
                //交易第二天开始
                if(assetsReporter.getDays()>1){
                    BigDecimal  thisDrawdownRate=BigDecimal.ONE.subtract(thisAssets.divide(upAssets,4,BigDecimal.ROUND_HALF_UP));
                    if(thisDrawdownRate.compareTo(maxDrawdownRate)==1){
                        maxDrawdownRate=thisDrawdownRate;
                    }
                }else{
                    //交易的第一天
                    if(assetsReporter.getDays()==1){
                        maxDrawdownRate=BigDecimal.ONE.subtract(thisAssets.divide(upAssets,4,BigDecimal.ROUND_HALF_UP));
                    }

                }
                upAssets=thisAssets;
            }
               return  maxDrawdownRate.multiply(BigDecimal.valueOf(100)).floatValue();
        } else {
            throw new EntityExistsException("assessUser:" + assessUserId);
        }
    }
}
