package com.blockchain.server.yyyf.indicators.handle;

import com.blockchain.server.yyyf.indicators.IndicatorsHandler;
import com.blockchain.server.yyyf.indicators.dto.AssetsReporter;
import com.blockchain.server.yyyf.service.AssetsReporterService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import javax.persistence.EntityExistsException;
import java.math.BigDecimal;
import java.util.List;

/**
 * @author fengyu
 */
@AllArgsConstructor
@Component(MaxLossRateIndicatorsHandler.HANDLER_NAME + IndicatorsHandler.HANDLER_NAME)
public class MaxLossRateIndicatorsHandler implements IndicatorsHandler {
    public static final String HANDLER_NAME = "MaxLossRate";
    private final AssetsReporterService assetsReporterService;

    @Override
    public Float handleIndicator(String assessUserId) {
        List<AssetsReporter> reporterList = assetsReporterService.selectAssetsReporterListByAssessUserId(assessUserId);

        if (reporterList != null && reporterList.size() > 0) {
            //如果期初资产
            AssetsReporter firstAssets = null;
            //交易期内资产最小值
            AssetsReporter minAssets = null;
            for (AssetsReporter assetsReporter : reporterList) {
                if (assetsReporter.getDays() == 0) { //如果期初资产
                    firstAssets = assetsReporter;
                } else {
                    if (minAssets == null) { //最小的若为null
                        minAssets = assetsReporter;
                    } else {
                        if (assetsReporter.getAssets().compareTo(minAssets.getAssets()) == -1) {
                            minAssets = assetsReporter;
                        }
                    }
                }
            }
            long minDay = minAssets.getDays();
            int n = (int) (360 / minDay);

            Float result = BigDecimal.ONE.subtract(minAssets.getAssets().divide(firstAssets.getAssets(), 5, BigDecimal.ROUND_HALF_UP).pow(n))
                    .setScale(2, BigDecimal.ROUND_HALF_UP).floatValue();

            //负无穷的时候为0
            if (result.isInfinite()) {
                result = Float.valueOf(0);
            }
            return result;

        } else {
            throw new EntityExistsException("assessUser:");
        }
    }
}
