package com.blockchain.server.yyyf.indicators.handle;

import com.blockchain.server.yyyf.indicators.IndicatorsHandler;
import com.blockchain.server.yyyf.indicators.dto.AssetsReporter;
import com.blockchain.server.yyyf.service.AssetsReporterService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import javax.persistence.EntityExistsException;
import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;

/**
 * @author fengyu
 */
@AllArgsConstructor
@Component(ProfitLossRateRateIndicatorsHandler.HANDLER_NAME + IndicatorsHandler.HANDLER_NAME)
public class ProfitLossRateRateIndicatorsHandler implements IndicatorsHandler {
    public static final String HANDLER_NAME = "ProfitLossRate";

    private final AssetsReporterService assetsReporterService;

    @Override
    public Float handleIndicator(String assessUserId) {

        List<AssetsReporter> reporterList = assetsReporterService.selectAssetsReporterListByAssessUserId(assessUserId);
        if (reporterList != null && reporterList.size() > 0) {
            //排序
            reporterList.sort(Comparator.comparingLong(AssetsReporter::getDays));
            //第一天（上一天）的资产总额
            BigDecimal upAssets = reporterList.get(0).getAssets();

            //总亏损额
            BigDecimal lossMoneyCount = BigDecimal.ZERO;
            //盈利计数
            int profitCount = 0;
            //总盈利额
            BigDecimal profitMoneyCount = BigDecimal.ZERO;
            for (AssetsReporter assetsReporter : reporterList) {

                BigDecimal thisYeid = assetsReporter.getAssets();
                //盈利
                if (thisYeid.compareTo(upAssets) > 0) { //盈利
                    profitCount++;
                    profitMoneyCount = profitMoneyCount.add(thisYeid).subtract(upAssets);
                }else{
                    lossMoneyCount = lossMoneyCount.add(thisYeid).subtract(upAssets);
                }
                upAssets=thisYeid;
            }
            if (profitCount == 0) { //每天亏损、或者每天既不盈利也不亏损
                return Float.valueOf(0);
            }else if(profitCount==(reporterList.size()-1)){ //每天都在盈利
                return Float.POSITIVE_INFINITY;
            }else { //计算盈亏比
                return profitMoneyCount.divide(lossMoneyCount,4,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
            }
        } else {
            throw new EntityExistsException("assessUser:" + assessUserId);
        }
    }
}
