package com.blockchain.server.yyyf.service.impl;

import com.blockchain.common.base.constant.YyyfConstant;
import com.blockchain.server.train.dto.TrainCaseTargetDto;
import com.blockchain.server.yyyf.dto.AssessUserDto;
import com.blockchain.server.yyyf.dto.ExamDto;
import com.blockchain.server.yyyf.dto.UserTimeMoneyDto;
import com.blockchain.server.yyyf.dto.YyyfStudentAnswerDto;
import com.blockchain.server.yyyf.entity.AssessUser;
import com.blockchain.server.yyyf.entity.AssessUserTarget;
import com.blockchain.server.yyyf.entity.YyyfMoney;
import com.blockchain.server.yyyf.enums.YyyfEnums;
import com.blockchain.server.yyyf.exceprion.YyyfException;
import com.blockchain.server.yyyf.indicators.IndicatorsManager;
import com.blockchain.server.yyyf.mapper.AssessUserMapper;
import com.blockchain.server.yyyf.mapper.AssessUserTargetMapper;
import com.blockchain.server.yyyf.mapper.YyyfMoneyMapper;
import com.blockchain.server.yyyf.service.AssessUserService;
import com.blockchain.server.yyyf.utils.HttpClientUtil;
import com.blockchain.server.yyyf.utils.IdGenerator;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author huan.xu
 * @version 1.0
 * @className AssessUserServiceImpl
 * @description 学生用户考核信息表 服务实现类
 * @date 2019-10-31 14:34
 */
@Transactional(rollbackFor = Exception.class)
@Service
public class AssessUserServiceImpl implements AssessUserService {
    private static Logger LOG = LoggerFactory.getLogger(AssessUserServiceImpl.class);

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private AssessUserMapper assessUserMapper;
    @Autowired
    private AssessUserTargetMapper assessUserTargetMapper;
    @Autowired
    private YyyfMoneyMapper yyyfMoneyMapper;

    //@Autowired
    private IndicatorsManager indicatorsManager;


    @Value("${yyyf.url}")
    private String yyyfUrl;


    @Value("${yyyf.assessment}")
    private String assessment;


    @Value("${yyyf.assessmentList}")
    private String assessmentList;


    @Override
    public AssessUser selectAssessUserById(String assessUserId) {

        return this.assessUserMapper.selectByPrimaryKey(assessUserId);
    }


    @Override
    public AssessUser selectAssessUserByUserIdAndissueIdForNotEnd(Integer userId, String issueId) {
        Example example = new Example(AssessUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andCondition(" user_id =", userId)
                .andCondition(" issue_id = ", issueId)
                .andCondition(" end_time  is null ");
        return this.assessUserMapper.selectOneByExample(example);
    }


    @Override
    public void insert(AssessUser assessUser) {
        this.assessUserMapper.insertSelective(assessUser);
    }

    @Override
    public void initAssessUser(AssessUser assessUser, List<TrainCaseTargetDto> trainCaseTargeList) {

        List<AssessUserTarget> list = new ArrayList<>(trainCaseTargeList.size());
        int j = 1;
        for (TrainCaseTargetDto trainCaseTargetDto : trainCaseTargeList) {
            AssessUserTarget assessUserTarget = new AssessUserTarget();
            BeanUtils.copyProperties(trainCaseTargetDto, assessUserTarget);
            assessUserTarget.setAssessUserId(assessUser.getId());
            assessUserTarget.setTargetId(trainCaseTargetDto.getId());
            assessUserTarget.setTargetName(trainCaseTargetDto.getName());
            assessUserTarget.setSort(j++);
            assessUserTarget.setAnswer(1);
            assessUserTarget.setId(IdGenerator.uuid());
            assessUserTarget.setStudentRate(0d);
            list.add(assessUserTarget);
        }

        //区块链原始金额存入
        YyyfMoney yyyfMoney = new YyyfMoney();
        yyyfMoney.setAssessUserId(assessUser.getId());
        yyyfMoney.setAliPay(assessUser.getAlipayInitMoney());
        yyyfMoney.setWxPay(assessUser.getWechatInitMoney());
        yyyfMoney.setBankCard(assessUser.getBankcardInitMoney());
        yyyfMoney.setBtc(BigDecimal.ZERO);
        yyyfMoney.setEth(BigDecimal.ZERO);
        yyyfMoney.setUsdt(BigDecimal.ZERO);
        yyyfMoney.setEos(BigDecimal.ZERO);
        yyyfMoney.setUpdateTime(new Date());

        this.assessUserMapper.insert(assessUser);
        this.yyyfMoneyMapper.insert(yyyfMoney);
        this.assessUserTargetMapper.batchInsert(list);
    }

    @Override
    public void update(AssessUser assessUser) {
        this.assessUserMapper.updateByPrimaryKey(assessUser);
    }


    @SneakyThrows
    @Override
    public AssessUserDto submit(String assessUserId) {
        AssessUser assessUser = this.selectAssessUserById(assessUserId);
        if (assessUser.getEndTime() != null) {
            throw new YyyfException(YyyfEnums.PROHIBIT_DUPLICATE_SUBMISSION);
        }
        Double totalScore = 0d;
        //获取指标信息
        Example example = new Example(AssessUserTarget.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andCondition(" assess_user_id = ", assessUserId);
        List<AssessUserTarget> assessUserTargetList = this.assessUserTargetMapper.selectByExample(example);

        //获取达标参数
        Map<String, Float> targetResultMap = indicatorsManager.calculatePageIndicators(assessUserId);
        //最终更新的指标库答题结果list
        List<AssessUserTarget> updateAssessUserTargetList = new ArrayList<>();

        for (AssessUserTarget assessUserTarget : assessUserTargetList) {
            //计算是否达标
            answerResultForTarget(assessUserTarget, targetResultMap);

            Double taskScore = assessUserTarget.getTaskScore();
            if (assessUserTarget.getAnswer() == 0) {
                totalScore += taskScore;
            }
            updateAssessUserTargetList.add(assessUserTarget);
        }

        Date startTime = assessUser.getStartTime();
        Date now = new Date();
        assessUser.setTotalScore(totalScore);
        assessUser.setEndTime(now);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //考试时长
        long duration = sdf.parse(sdf.format(now)).getTime() - sdf.parse(sdf.format(startTime)).getTime();
        assessUser.setDuration(Math.toIntExact(duration));
        if (assessUser.getUserType() == 0 && assessUser.getAchieveMentType() == 0) {
            YyyfStudentAnswerDto yyyfStudentAnswerDto = new YyyfStudentAnswerDto();
            BeanUtils.copyProperties(assessUser, yyyfStudentAnswerDto);
            yyyfStudentAnswerDto.setAssessmentId(assessUser.getIssueId());
            HttpClientUtil.doPostWithJSON(new StringBuilder().append(yyyfUrl).append(assessment).toString(), yyyfStudentAnswerDto);
        }

        this.update(assessUser);
        //批量更新指标最终结果
        this.assessUserTargetMapper.batchUpdateTargetAnswer(updateAssessUserTargetList);

        AssessUserDto assessUserDto = new AssessUserDto();
        BeanUtils.copyProperties(assessUser, assessUserDto);
        assessUserDto.setAssessUserTargetList(assessUserTargetList);

        return assessUserDto;
    }

    @SneakyThrows
    @Override
    public void synchronousScore(String issueId) {
        List<AssessUserDto> assessUserDtoList = this.assessUserMapper.selectAssessUserDtoByIssueId(issueId);
        List<AssessUserDto> updateAssessUserDtoList = new ArrayList<>();
        List<YyyfStudentAnswerDto> yyyfStudentAnswerDtoList = new ArrayList<>(assessUserDtoList.size());
        //最终更新的指标库答题结果list
        List<AssessUserTarget> updateAssessUserTargetList = new ArrayList<>();
        Date now = new Date();

        for (AssessUserDto assessUserDto : assessUserDtoList) {
            //根据assessUserId 获取当时指标情况
            Map<String, Float> targetResultMap = indicatorsManager.calculatePageIndicators(assessUserDto.getId());
            List<AssessUserTarget> assessUserTargetList = assessUserDto.getAssessUserTargetList();
            if (assessUserDto.getEndTime() == null) {
                Double totalScore = 0d;
                for (AssessUserTarget assessUserTarget : assessUserTargetList) {
                    //计算是否达标
                    answerResultForTarget(assessUserTarget, targetResultMap);
                    //计算成绩
                    Double taskScore = assessUserTarget.getTaskScore();
                    if (assessUserTarget.getAnswer() == 0) {
                        totalScore += taskScore;
                    }
                    updateAssessUserTargetList.add(assessUserTarget);
                }
                Date startTime = assessUserDto.getStartTime();
                assessUserDto.setTotalScore(totalScore);
                assessUserDto.setEndTime(now);
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                //考试时长
                long duration = sdf.parse(sdf.format(now)).getTime() - sdf.parse(sdf.format(startTime)).getTime();
                assessUserDto.setDuration(Math.toIntExact(duration));
                updateAssessUserDtoList.add(assessUserDto);
                //指标库更新

            }
            YyyfStudentAnswerDto yyyfStudentAnswerDto = new YyyfStudentAnswerDto();
            BeanUtils.copyProperties(assessUserDto, yyyfStudentAnswerDto);
            yyyfStudentAnswerDto.setAssessmentId(issueId);
            yyyfStudentAnswerDtoList.add(yyyfStudentAnswerDto);
            redisTemplate.delete(YyyfConstant.CASE_KEY + assessUserDto.getUserId());
        }

        //获取考核信息
        HashOperations<String, String, ExamDto> examDtoOpsForHash = redisTemplate.opsForHash();
        // 获取所有参加考核学生信息
        HashOperations<String, String, List<String>> studentOpsForHash = redisTemplate.opsForHash();
        studentOpsForHash.delete(YyyfConstant.JOIN_EXAM_STU_IDS_KEY, issueId);
        examDtoOpsForHash.delete(YyyfConstant.EXAM_KEY, issueId);

        if (updateAssessUserDtoList.size() > 0) {
            this.assessUserMapper.batchUpdateTotalScoreByAssessUserDtoList(assessUserDtoList);
            this.assessUserTargetMapper.batchUpdateTargetAnswer(updateAssessUserTargetList);
        }

        HttpClientUtil.doPostWithJSON(new StringBuilder().append(yyyfUrl).append(assessmentList).toString(), yyyfStudentAnswerDtoList);

    }

    @Override
    public AssessUser selectScore(String issueId, Integer userId) {
        Example example = new Example(AssessUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andCondition(" user_id =", userId)
                .andCondition(" issue_id = ", issueId)
                .andCondition(" end_time  is not null ");
        return this.assessUserMapper.selectOneByExample(example);
    }

    @Override
    public int selectCountByByIssueId(String issueId) {
        Example example = new Example(AssessUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andCondition("issue_id = ", issueId);
        return this.assessUserMapper.selectCountByExample(example);
    }

    @Override
    public AssessUser getAssessUserByIssueIdAndUserIdForExam(String issueId, Integer userId) {
        Example example = new Example(AssessUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andCondition("issue_id = ", issueId)
                .andCondition("user_id = ", userId);
        return this.assessUserMapper.selectOneByExample(example);
    }

    @Override
    public List<AssessUser> getAssessUserListByIssueIdAndUserId(String issueId, Integer userId) {
        Example example = new Example(AssessUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andCondition("issue_id = ", issueId)
                .andCondition("user_id = ", userId);
        return this.assessUserMapper.selectByExample(example);
    }

    @Override
    public AssessUser selectById(String assessUserId) {
        return this.assessUserMapper.selectByPrimaryKey(assessUserId);
    }

    @Override
    public List<UserTimeMoneyDto> selectUserTimeMoneyDtosForNotEnd() {
        return this.assessUserMapper.selectUserTimeMoneyDtosForNotEnd();
    }

    @Override
    public UserTimeMoneyDto getUserTimeMoneyDtoByAssessUserId(String assessUserId) {
        return this.assessUserMapper.getUserTimeMoneyDtoByAssessUserId(assessUserId);
    }

    /**
     * @param [assessUserTarget, targetResultMap]
     * @return void
     * @description 计算是否达标
     * @author Mr.Xu
     * @date 2020-06-01 22:49:37
     **/
    private void answerResultForTarget(AssessUserTarget assessUserTarget, Map<String, Float> targetResultMap) {
        assessUserTarget.setAnswer(1);
        Float aFloat = targetResultMap.get(assessUserTarget.getCode());
        Double taskRate = 0d;
        if (aFloat != null) {
            taskRate = Double.valueOf(aFloat);
            assessUserTarget.setStudentRate(taskRate);
            int compareAnswer = taskRate.compareTo(assessUserTarget.getTaskRate());
            //最终指标结果一致
            if (assessUserTarget.getTaskOperate().intValue() == compareAnswer) {
                assessUserTarget.setAnswer(0);
            }
        }

    }

}