package com.blockchain.server.yyyf.service.impl;

import com.blockchain.server.yyyf.indicators.dto.AssetsReporter;
import com.blockchain.server.yyyf.mapper.AssetsReporterMapper;
import com.blockchain.server.yyyf.service.AssetsReporterService;
import lombok.AllArgsConstructor;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

/**
 * @author fengyu
 */
@Service
@AllArgsConstructor
public class AssetsReporterServiceImpl implements AssetsReporterService {
    private final AssetsReporterMapper assetsReporterMapper;
    private final CacheManager cacheManager;

    public static final String CACHE_ASSETS_REPORTER = "yyyf_assets_reporter";
    public static final String CACHE_KEY_ASSETS_REPORTER = "yyyf:assetsReporter:";

    @Override
    public void saveList(List<AssetsReporter> assetsReporterList) {
        //TODO  Transaction rollback, retry, log
        assetsReporterList.forEach(assetsReporter ->
                Optional.ofNullable(cacheManager.getCache(CACHE_ASSETS_REPORTER))
                        .ifPresent(cache -> cache.evict(CACHE_KEY_ASSETS_REPORTER.concat(assetsReporter.getAssessUserId()))));
        assetsReporterMapper.insertBatch(assetsReporterList);
    }

    @Override
    @Cacheable(cacheNames = CACHE_ASSETS_REPORTER,
            key = "#root.target.CACHE_KEY_ASSETS_REPORTER.concat(#assessUserId)", unless = "#result == null")
    public List<AssetsReporter> selectAssetsReporterListByAssessUserId(String assessUserId) {
        return assetsReporterMapper.selectByAssessUserId(assessUserId);
    }
}
