package com.blockchain.server.yyyf.task;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.blockchain.server.yyyf.dto.UserTimeMoneyDto;
import com.blockchain.server.yyyf.indicators.IndicatorsManager;
import com.blockchain.server.yyyf.indicators.dto.AssetsReporter;
import com.blockchain.server.yyyf.service.AssessUserService;
import com.blockchain.server.yyyf.service.AssetsReporterService;
import com.blockchain.server.yyyf.utils.IdGenerator;
import lombok.AllArgsConstructor;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @author fengyu
 */
@Configuration
@EnableScheduling
@AllArgsConstructor
public class ExamProcessTask {

    private final AssessUserService assessUserService;
    private final IndicatorsManager indicatorsManager;
    private final AssetsReporterService assetsReporterService;

    @Scheduled(cron = "0 1 0 * * ?")
    public void processRecording() {
        List<UserTimeMoneyDto> userTimeMoneyList = this.assessUserService.selectUserTimeMoneyDtosForNotEnd();
        List<AssetsReporter> assetsReporterList = new ArrayList<>(userTimeMoneyList.size());
        DateTime now = DateUtil.date();
        userTimeMoneyList.forEach(entity -> {
            BigDecimal assets = indicatorsManager.calculateCurrentTotalAssets(entity.getQUserId(), entity.getAssessUserId());
            assetsReporterList.add(AssetsReporter.builder()
                    .id(IdGenerator.uuid())
                    .assessUserId(entity.getAssessUserId())
                    .assets(assets)
                    .days(DateUtil.between(entity.getStartTime(), now, DateUnit.DAY))
                    .coinName("RMB")
                    .createTime(now)
                    .build());
        });
        if (!assetsReporterList.isEmpty()) {
            assetsReporterService.saveList(assetsReporterList);
        }
    }
}
